@@ -1,9 +1,11 @@
 """Calculate graph edge bearings."""
 
-import matplotlib.pyplot as plt
+from warnings import warn
+
 import networkx as nx
 import numpy as np
 
+from . import plot
 from . import projection
 
 # scipy is an optional dependency for entropy calculation
@@ -13,24 +15,24 @@ except ImportError:  # pragma: no cover
     scipy = None
 
 
-def calculate_bearing(lat1, lng1, lat2, lng2):
+def calculate_bearing(lat1, lon1, lat2, lon2):
     """
-    Calculate the compass bearing(s) between pairs of lat-lng points.
+    Calculate the compass bearing(s) between pairs of lat-lon points.
 
-    Vectorized function to calculate (initial) bearings between two points'
+    Vectorized function to calculate initial bearings between two points'
     coordinates or between arrays of points' coordinates. Expects coordinates
-    in decimal degrees. Bearing represents angle in degrees (clockwise)
-    between north and the geodesic line from point 1 to point 2.
+    in decimal degrees. Bearing represents the clockwise angle in degrees
+    between north and the geodesic line from (lat1, lon1) to (lat2, lon2).
 
     Parameters
     ----------
     lat1 : float or numpy.array of float
         first point's latitude coordinate
-    lng1 : float or numpy.array of float
+    lon1 : float or numpy.array of float
         first point's longitude coordinate
     lat2 : float or numpy.array of float
         second point's latitude coordinate
-    lng2 : float or numpy.array of float
+    lon2 : float or numpy.array of float
         second point's longitude coordinate
 
     Returns
@@ -38,21 +40,21 @@ def calculate_bearing(lat1, lng1, lat2, lng2):
     bearing : float or numpy.array of float
         the bearing(s) in decimal degrees
     """
-    # get the latitudes and the difference in longitudes, in radians
+    # get the latitudes and the difference in longitudes, all in radians
     lat1 = np.radians(lat1)
     lat2 = np.radians(lat2)
-    d_lng = np.radians(lng2 - lng1)
+    delta_lon = np.radians(lon2 - lon1)
 
     # calculate initial bearing from -180 degrees to +180 degrees
-    y = np.sin(d_lng) * np.cos(lat2)
-    x = np.cos(lat1) * np.sin(lat2) - np.sin(lat1) * np.cos(lat2) * np.cos(d_lng)
+    y = np.sin(delta_lon) * np.cos(lat2)
+    x = np.cos(lat1) * np.sin(lat2) - np.sin(lat1) * np.cos(lat2) * np.cos(delta_lon)
     initial_bearing = np.degrees(np.arctan2(y, x))
 
     # normalize to 0-360 degrees to get compass bearing
     return initial_bearing % 360
 
 
-def add_edge_bearings(G, precision=1):
+def add_edge_bearings(G, precision=None):
     """
     Add compass `bearing` attributes to all graph edges.
 
@@ -68,15 +70,25 @@ def add_edge_bearings(G, precision=1):
     G : networkx.MultiDiGraph
         unprojected graph
     precision : int
-        decimal precision to round bearing
+        deprecated, do not use
 
     Returns
     -------
     G : networkx.MultiDiGraph
         graph with edge bearing attributes
     """
+    if precision is None:
+        precision = 1
+    else:
+        warn(
+            "The `precision` parameter is deprecated and will be removed in the v2.0.0 release.",
+            FutureWarning,
+            stacklevel=2,
+        )
+
     if projection.is_projected(G.graph["crs"]):  # pragma: no cover
-        raise ValueError("graph must be unprojected to add edge bearings")
+        msg = "graph must be unprojected to add edge bearings"
+        raise ValueError(msg)
 
     # extract edge IDs and corresponding coordinates from their nodes
     uvk = [(u, v, k) for u, v, k in G.edges if u != v]
@@ -100,13 +112,17 @@ def orientation_entropy(Gu, num_bins=36, min_length=0, weight=None):
     across evenly spaced bins. Ignores self-loop edges as their bearings are
     undefined.
 
+    For more info see: Boeing, G. 2019. "Urban Spatial Order: Street Network
+    Orientation, Configuration, and Entropy." Applied Network Science, 4 (1),
+    67. https://doi.org/10.1007/s41109-019-0189-1
+
     Parameters
     ----------
     Gu : networkx.MultiGraph
         undirected, unprojected graph with `bearing` attributes on each edge
     num_bins : int
         number of bins; for example, if `num_bins=36` is provided, then each
-        bin will represent 10° around the compass
+        bin will represent 10 degrees around the compass
     min_length : float
         ignore edges with `length` attributes less than `min_length`; useful
         to ignore the noise of many very short edges
@@ -123,7 +139,8 @@ def orientation_entropy(Gu, num_bins=36, min_length=0, weight=None):
     """
     # check if we were able to import scipy
     if scipy is None:  # pragma: no cover
-        raise ImportError("scipy must be installed to calculate entropy")
+        msg = "scipy must be installed to calculate entropy"
+        raise ImportError(msg)
     bin_counts, _ = _bearings_distribution(Gu, num_bins, min_length, weight)
     return scipy.stats.entropy(bin_counts)
 
@@ -132,8 +149,8 @@ def _extract_edge_bearings(Gu, min_length=0, weight=None):
     """
     Extract undirected graph's bidirectional edge bearings.
 
-    For example, if an edge has a bearing of 90° then we will record bearings
-    of both 90° and 270° for this edge.
+    For example, if an edge has a bearing of 90 degrees then we will record
+    bearings of both 90 degrees and 270 degrees for this edge.
 
     Parameters
     ----------
@@ -154,8 +171,9 @@ def _extract_edge_bearings(Gu, min_length=0, weight=None):
         the graph's bidirectional edge bearings
     """
     if nx.is_directed(Gu) or projection.is_projected(Gu.graph["crs"]):  # pragma: no cover
-        raise ValueError("graph must be undirected and unprojected to analyze edge bearings")
-    bearings = list()
+        msg = "graph must be undirected and unprojected to analyze edge bearings"
+        raise ValueError(msg)
+    bearings = []
     for u, v, data in Gu.edges(data=True):
         # ignore self-loops and any edges below min_length
         if u != v and data["length"] >= min_length:
@@ -177,10 +195,11 @@ def _bearings_distribution(Gu, num_bins, min_length=0, weight=None):
     """
     Compute distribution of bearings across evenly spaced bins.
 
-    Prevents bin-edge effects around common values like 0° and 90° by
-    initially creating twice as many bins as desired, then merging them in
-    pairs. For example, if `num_bins=36` is provided, then each bin will
-    represent 10° around the compass, with the first bin representing 355°-5°.
+    Prevents bin-edge effects around common values like 0 degrees and 90
+    degrees by initially creating twice as many bins as desired, then merging
+    them in pairs. For example, if `num_bins=36` is provided, then each bin
+    will represent 10 degrees around the compass, with the first bin
+    representing 355 degrees to 5 degrees.
 
     Parameters
     ----------
@@ -208,7 +227,8 @@ def _bearings_distribution(Gu, num_bins, min_length=0, weight=None):
     bearings = _extract_edge_bearings(Gu, min_length, weight)
     count, bin_edges = np.histogram(bearings, bins=bins)
 
-    # move last bin to front, so eg 0.01° and 359.99° will be binned together
+    # move last bin to front, so eg 0.01 degrees and 359.99 degrees will be
+    # binned together
     count = np.roll(count, 1)
     bin_counts = count[::2] + count[1::2]
 
@@ -235,118 +255,69 @@ def plot_orientation(
     xtick_font=None,
 ):
     """
-    Plot a polar histogram of a spatial network's bidirectional edge bearings.
+    Do not use: deprecated.
 
-    Ignores self-loop edges as their bearings are undefined.
-
-    For more info see: Boeing, G. 2019. "Urban Spatial Order: Street Network
-    Orientation, Configuration, and Entropy." Applied Network Science, 4 (1),
-    67. https://doi.org/10.1007/s41109-019-0189-1
+    The plot_orientation function moved to the plot module. Calling it via the
+    bearing module will raise an error starting in the v2.0.0 release.
 
     Parameters
     ----------
     Gu : networkx.MultiGraph
-        undirected, unprojected graph with `bearing` attributes on each edge
+        deprecated, do not use
     num_bins : int
-        number of bins; for example, if `num_bins=36` is provided, then each
-        bin will represent 10° around the compass
+        deprecated, do not use
     min_length : float
-        ignore edges with `length` attributes less than `min_length`
+        deprecated, do not use
     weight : string
-        if not None, weight edges' bearings by this (non-null) edge attribute
+        deprecated, do not use
     ax : matplotlib.axes.PolarAxesSubplot
-        if not None, plot on this preexisting axis; must have projection=polar
+        deprecated, do not use
     figsize : tuple
-        if ax is None, create new figure with size (width, height)
+        deprecated, do not use
     area : bool
-        if True, set bar length so area is proportional to frequency,
-        otherwise set bar length so height is proportional to frequency
+        deprecated, do not use
     color : string
-        color of histogram bars
+        deprecated, do not use
     edgecolor : string
-        color of histogram bar edges
+        deprecated, do not use
     linewidth : float
-        width of histogram bar edges
+        deprecated, do not use
     alpha : float
-        opacity of histogram bars
+        deprecated, do not use
     title : string
-        title for plot
+        deprecated, do not use
     title_y : float
-        y position to place title
+        deprecated, do not use
     title_font : dict
-        the title's fontdict to pass to matplotlib
+        deprecated, do not use
     xtick_font : dict
-        the xtick labels' fontdict to pass to matplotlib
+        deprecated, do not use
 
     Returns
     -------
     fig, ax : tuple
         matplotlib figure, axis
     """
-    if title_font is None:
-        title_font = {"family": "DejaVu Sans", "size": 24, "weight": "bold"}
-    if xtick_font is None:
-        xtick_font = {
-            "family": "DejaVu Sans",
-            "size": 10,
-            "weight": "bold",
-            "alpha": 1.0,
-            "zorder": 3,
-        }
-
-    # get the bearings' distribution's bin counts and edges
-    bin_counts, bin_edges = _bearings_distribution(Gu, num_bins, min_length, weight)
-
-    # positions: where to center each bar. ignore the last bin edge, because
-    # it's the same as the first (i.e., 0° = 360°)
-    positions = np.radians(bin_edges[:-1])
-
-    # width: make bars fill the circumference without gaps or overlaps
-    width = 2 * np.pi / num_bins
-
-    # radius: how long to make each bar
-    bin_frequency = bin_counts / bin_counts.sum()
-    if area:
-        # set bar length so area is proportional to frequency
-        radius = np.sqrt(bin_frequency)
-    else:
-        # set bar length so height is proportional to frequency
-        radius = bin_frequency
-
-    # create ax (if necessary) then set N at top and go clockwise
-    if ax is None:
-        fig, ax = plt.subplots(figsize=figsize, subplot_kw={"projection": "polar"})
-    else:
-        fig = ax.figure
-    ax.set_theta_zero_location("N")
-    ax.set_theta_direction("clockwise")
-    ax.set_ylim(top=radius.max())
-
-    # configure the y-ticks and remove their labels
-    ax.set_yticks(np.linspace(0, radius.max(), 5))
-    ax.set_yticklabels(labels="")
-
-    # configure the x-ticks and their labels
-    xticklabels = ["N", "", "E", "", "S", "", "W", ""]
-    ax.set_xticks(ax.get_xticks())
-    ax.set_xticklabels(labels=xticklabels, fontdict=xtick_font)
-    ax.tick_params(axis="x", which="major", pad=-2)
-
-    # draw the bars
-    ax.bar(
-        positions,
-        height=radius,
-        width=width,
-        align="center",
-        bottom=0,
-        zorder=2,
+    warn(
+        "The `plot_orientation` function moved to the `plot` module. Calling it "
+        "via the `bearing` module will raise an exception starting with the v2.0.0 release.",
+        FutureWarning,
+        stacklevel=2,
+    )
+    return plot.plot_orientation(
+        Gu,
+        num_bins=num_bins,
+        min_length=min_length,
+        weight=weight,
+        ax=ax,
+        figsize=figsize,
+        area=area,
         color=color,
         edgecolor=edgecolor,
         linewidth=linewidth,
         alpha=alpha,
+        title=title,
+        title_y=title_y,
+        title_font=title_font,
+        xtick_font=xtick_font,
     )
-
-    if title:
-        ax.set_title(title, y=title_y, fontdict=title_font)
-    fig.tight_layout()
-    return fig, ax
