@@ -9,10 +9,13 @@ only) by passing a dictionary of desired OSM tags.
 
 For more details, see https://wiki.openstreetmap.org/wiki/Map_features and
 https://wiki.openstreetmap.org/wiki/Elements
+
+Refer to the Getting Started guide for usage limitations.
 """
 
 import logging as lg
 import warnings
+from warnings import warn
 
 import geopandas as gpd
 import pandas as pd
@@ -25,13 +28,14 @@ from shapely.geometry import Polygon
 from shapely.ops import linemerge
 from shapely.ops import polygonize
 
-from . import _downloader
+from . import _overpass
 from . import geocoder
 from . import osm_xml
 from . import settings
 from . import utils
 from . import utils_geo
-from ._errors import EmptyOverpassResponse
+from ._errors import CacheOnlyInterruptError
+from ._errors import InsufficientResponseError
 
 # dict of tags to determine if closed ways should be polygons, based on JSON
 # from https://wiki.openstreetmap.org/wiki/Overpass_turbo/Polygon_Features
@@ -75,7 +79,7 @@ _POLYGON_FEATURES = {
 }
 
 
-def features_from_bbox(north, south, east, west, tags):
+def features_from_bbox(north=None, south=None, east=None, west=None, bbox=None, tags=None):
     """
     Create a GeoDataFrame of OSM features within a N, S, E, W bounding box.
 
@@ -88,15 +92,17 @@ def features_from_bbox(north, south, east, west, tags):
     Parameters
     ----------
     north : float
-        northern latitude of bounding box
+        deprecated, do not use
     south : float
-        southern latitude of bounding box
+        deprecated, do not use
     east : float
-        eastern longitude of bounding box
+        deprecated, do not use
     west : float
-        western longitude of bounding box
+        deprecated, do not use
+    bbox : tuple of floats
+        bounding box as (north, south, east, west)
     tags : dict
-        Dict of tags used for finding objects in the selected area. Results
+        Dict of tags used for finding elements in the selected area. Results
         returned are the union, not intersection of each individual tag.
         Each result matches at least one given tag. The dict keys should be
         OSM tags, (e.g., `building`, `landuse`, `highway`, etc) and the dict
@@ -112,13 +118,19 @@ def features_from_bbox(north, south, east, west, tags):
     -------
     gdf : geopandas.GeoDataFrame
     """
+    if not (north is None and south is None and east is None and west is None):
+        msg = (
+            "The `north`, `south`, `east`, and `west` parameters are deprecated and "
+            "will be removed in the v2.0.0 release. Use the `bbox` parameter instead."
+        )
+        warn(msg, FutureWarning, stacklevel=2)
+        bbox = (north, south, east, west)
+
     # convert bounding box to a polygon
-    polygon = utils_geo.bbox_to_poly(north, south, east, west)
+    polygon = utils_geo.bbox_to_poly(bbox=bbox)
 
     # create GeoDataFrame of features within this polygon
-    gdf = features_from_polygon(polygon, tags)
-
-    return gdf
+    return features_from_polygon(polygon, tags)
 
 
 def features_from_point(center_point, tags, dist=1000):
@@ -134,9 +146,9 @@ def features_from_point(center_point, tags, dist=1000):
     Parameters
     ----------
     center_point : tuple
-        the (lat, lng) center point around which to get the features
+        the (lat, lon) center point around which to get the features
     tags : dict
-        Dict of tags used for finding objects in the selected area. Results
+        Dict of tags used for finding elements in the selected area. Results
         returned are the union, not intersection of each individual tag.
         Each result matches at least one given tag. The dict keys should be
         OSM tags, (e.g., `building`, `landuse`, `highway`, etc) and the dict
@@ -155,15 +167,13 @@ def features_from_point(center_point, tags, dist=1000):
     gdf : geopandas.GeoDataFrame
     """
     # create bounding box from center point and distance in each direction
-    north, south, east, west = utils_geo.bbox_from_point(center_point, dist)
+    bbox = utils_geo.bbox_from_point(center_point, dist)
 
     # convert the bounding box to a polygon
-    polygon = utils_geo.bbox_to_poly(north, south, east, west)
+    polygon = utils_geo.bbox_to_poly(bbox=bbox)
 
     # create GeoDataFrame of features within this polygon
-    gdf = features_from_polygon(polygon, tags)
-
-    return gdf
+    return features_from_polygon(polygon, tags)
 
 
 def features_from_address(address, tags, dist=1000):
@@ -182,7 +192,7 @@ def features_from_address(address, tags, dist=1000):
         the address to geocode and use as the central point around which to
         get the features
     tags : dict
-        Dict of tags used for finding objects in the selected area. Results
+        Dict of tags used for finding elements in the selected area. Results
         returned are the union, not intersection of each individual tag.
         Each result matches at least one given tag. The dict keys should be
         OSM tags, (e.g., `building`, `landuse`, `highway`, etc) and the dict
@@ -200,13 +210,11 @@ def features_from_address(address, tags, dist=1000):
     -------
     gdf : geopandas.GeoDataFrame
     """
-    # geocode the address string to a (lat, lng) point
+    # geocode the address string to a (lat, lon) point
     center_point = geocoder.geocode(query=address)
 
     # create GeoDataFrame of features around this point
-    gdf = features_from_point(center_point, tags, dist=dist)
-
-    return gdf
+    return features_from_point(center_point, tags, dist=dist)
 
 
 def features_from_place(query, tags, which_result=None, buffer_dist=None):
@@ -237,7 +245,7 @@ def features_from_place(query, tags, which_result=None, buffer_dist=None):
     query : string or dict or list
         the query or queries to geocode to get place boundary polygon(s)
     tags : dict
-        Dict of tags used for finding objects in the selected area. Results
+        Dict of tags used for finding elements in the selected area. Results
         returned are the union, not intersection of each individual tag.
         Each result matches at least one given tag. The dict keys should be
         OSM tags, (e.g., `building`, `landuse`, `highway`, etc) and the dict
@@ -252,12 +260,20 @@ def features_from_place(query, tags, which_result=None, buffer_dist=None):
         which geocoding result to use. if None, auto-select the first
         (Multi)Polygon or raise an error if OSM doesn't return one.
     buffer_dist : float
-        distance to buffer around the place geometry, in meters
+        deprecated, do not use
 
     Returns
     -------
     gdf : geopandas.GeoDataFrame
     """
+    if buffer_dist is not None:
+        warn(
+            "The buffer_dist argument has been deprecated and will be removed "
+            "in the v2.0.0 release. Buffer your query area directly, if desired.",
+            FutureWarning,
+            stacklevel=2,
+        )
+
     # create a GeoDataFrame with the spatial boundaries of the place(s)
     if isinstance(query, (str, dict)):
         # if it is a string (place name) or dict (structured place query),
@@ -269,16 +285,15 @@ def features_from_place(query, tags, which_result=None, buffer_dist=None):
         # if it is a list, it contains multiple places to get
         gdf_place = geocoder.geocode_to_gdf(query, buffer_dist=buffer_dist)
     else:  # pragma: no cover
-        raise TypeError("query must be dict, string, or list of strings")
+        msg = "query must be dict, string, or list of strings"
+        raise TypeError(msg)
 
     # extract the geometry from the GeoDataFrame to use in API query
     polygon = gdf_place["geometry"].unary_union
     utils.log("Constructed place geometry polygon(s) to query API")
 
     # create GeoDataFrame using this polygon(s) geometry
-    gdf = features_from_polygon(polygon, tags)
-
-    return gdf
+    return features_from_polygon(polygon, tags)
 
 
 def features_from_polygon(polygon, tags):
@@ -296,7 +311,7 @@ def features_from_polygon(polygon, tags):
     polygon : shapely.geometry.Polygon or shapely.geometry.MultiPolygon
         geographic boundaries to fetch features within
     tags : dict
-        Dict of tags used for finding objects in the selected area. Results
+        Dict of tags used for finding elements in the selected area. Results
         returned are the union, not intersection of each individual tag.
         Each result matches at least one given tag. The dict keys should be
         OSM tags, (e.g., `building`, `landuse`, `highway`, etc) and the dict
@@ -314,25 +329,25 @@ def features_from_polygon(polygon, tags):
     """
     # verify that the geometry is valid and a Polygon/MultiPolygon
     if not polygon.is_valid:
-        raise ValueError("The geometry of `polygon` is invalid")
+        msg = "The geometry of `polygon` is invalid"
+        raise ValueError(msg)
     if not isinstance(polygon, (Polygon, MultiPolygon)):
-        raise TypeError(
-            "Boundaries must be a shapely Polygon or MultiPolygon. If you requested "
-            "features from place name, make sure your query resolves to a Polygon or "
-            "MultiPolygon, and not some other geometry, like a Point. See OSMnx "
-            "documentation for details."
+        msg = (
+            "Boundaries must be a shapely Polygon or MultiPolygon. If you "
+            "requested features from place name, make sure your query resolves "
+            "to a Polygon or MultiPolygon, and not some other geometry, like a "
+            "Point. See OSMnx documentation for details."
         )
+        raise TypeError(msg)
 
     # download the data from OSM
-    response_jsons = _downloader._osm_features_download(polygon, tags)
+    response_jsons = _overpass._download_overpass_features(polygon, tags)
 
     # create GeoDataFrame from the downloaded data
-    gdf = _create_gdf(response_jsons, polygon, tags)
-
-    return gdf
+    return _create_gdf(response_jsons, polygon, tags)
 
 
-def features_from_xml(filepath, polygon=None, tags=None):
+def features_from_xml(filepath, polygon=None, tags=None, encoding="utf-8"):
     """
     Create a GeoDataFrame of OSM features in an OSM-formatted XML file.
 
@@ -350,9 +365,9 @@ def features_from_xml(filepath, polygon=None, tags=None):
     filepath : string or pathlib.Path
         path to file containing OSM XML data
     polygon : shapely.geometry.Polygon
-        optional geographic boundary to filter objects
+        optional geographic boundary to filter elements
     tags : dict
-        optional dict of tags for filtering objects from the XML. Results
+        optional dict of tags for filtering elements from the XML. Results
         returned are the union, not intersection of each individual tag.
         Each result matches at least one given tag. The dict keys should be
         OSM tags, (e.g., `building`, `landuse`, `highway`, etc) and the dict
@@ -363,18 +378,18 @@ def features_from_xml(filepath, polygon=None, tags=None):
         the area. `tags = {'amenity':True, 'landuse':['retail','commercial'],
         'highway':'bus_stop'}` would return all amenities, landuse=retail,
         landuse=commercial, and highway=bus_stop.
+    encoding : string
+        the XML file's character encoding
 
     Returns
     -------
     gdf : geopandas.GeoDataFrame
     """
     # transmogrify file of OSM XML data into JSON
-    response_jsons = [osm_xml._overpass_json_from_file(filepath)]
+    response_jsons = [osm_xml._overpass_json_from_file(filepath, encoding)]
 
     # create GeoDataFrame using this response JSON
-    gdf = _create_gdf(response_jsons, polygon=polygon, tags=tags)
-
-    return gdf
+    return _create_gdf(response_jsons, polygon=polygon, tags=tags)
 
 
 def _create_gdf(response_jsons, polygon, tags):
@@ -400,17 +415,14 @@ def _create_gdf(response_jsons, polygon, tags):
     gdf : geopandas.GeoDataFrame
         GeoDataFrame of features and their associated tags
     """
-    elements_count = sum(len(rj["elements"]) for rj in response_jsons)
-
-    # make sure we got data back from the server request(s)
-    if elements_count == 0:
-        msg = (
-            "There are no data elements in the server response. Check log and query location/tags."
-        )
-        raise EmptyOverpassResponse(msg)
-
-    # begin processing the elements retrieved from the server
-    utils.log(f"Converting {elements_count} elements in JSON responses to features")
+    response_count = 0
+    if settings.cache_only_mode:
+        # if cache_only_mode, consume response_jsons then interrupt
+        for _ in response_jsons:
+            response_count += 1
+        utils.log(f"Retrieved all data from API in {response_count} request(s)")
+        msg = "Interrupted because `settings.cache_only_mode=True`"
+        raise CacheOnlyInterruptError(msg)
 
     # Dictionaries to hold nodes and complete geometries
     coords = {}
@@ -424,6 +436,8 @@ def _create_gdf(response_jsons, polygon, tags):
 
     # extract geometries from the downloaded osm data
     for response_json in response_jsons:
+        response_count += 1
+
         # Parses the JSON of OSM nodes, ways and (multipolygon) relations
         # to dictionaries of coordinates, Shapely Points, LineStrings,
         # Polygons and MultiPolygons
@@ -465,12 +479,17 @@ def _create_gdf(response_jsons, polygon, tags):
                 )
                 geometries[unique_id] = multipolygon
 
-    utils.log(f"{len(geometries)} geometries created in the dict")
+    utils.log(f"Retrieved all data from API in {response_count} request(s)")
+
+    # ensure we got some node/way data back from the server request(s)
+    if len(geometries) == 0:  # pragma: no cover
+        msg = "No data elements in server response. Check log and query location/tags."
+        raise InsufficientResponseError(msg)
 
     # remove untagged elements from the final dict of geometries
+    utils.log(f"{len(geometries)} geometries created in the dict")
     for untagged_element_id in untagged_element_ids:
         geometries.pop(untagged_element_id, None)
-
     utils.log(f"{len(untagged_element_ids)} untagged features removed")
 
     # create GeoDataFrame, ensure it has geometry, then set crs
@@ -517,8 +536,7 @@ def _parse_node_to_coords(element):
         dict of latitude/longitude coordinates
     """
     # return the coordinate of a single node element
-    coords = {"lat": element["lat"], "lon": element["lon"]}
-    return coords
+    return {"lat": element["lat"], "lon": element["lon"]}
 
 
 def _parse_node_to_point(element):
@@ -549,7 +567,7 @@ def _parse_node_to_point(element):
     return point
 
 
-def _parse_way_to_linestring_or_polygon(element, coords, polygon_features=_POLYGON_FEATURES):
+def _parse_way_to_linestring_or_polygon(element, coords):
     """
     Parse open LineString, closed LineString or Polygon from OSM 'way'.
 
@@ -562,8 +580,6 @@ def _parse_way_to_linestring_or_polygon(element, coords, polygon_features=_POLYG
         element type "way" from overpass response JSON
     coords : dict
         dict of node IDs and their latitude/longitude coordinates
-    polygon_features : dict
-        dict for determining whether closed ways are LineStrings or Polygons
 
     Returns
     -------
@@ -705,12 +721,12 @@ def _is_closed_way_a_polygon(element, polygon_features=_POLYGON_FEATURES):
                         is_polygon = True
 
                 # if the key is for a passlist i.e. specific tags should
-                # become Polygons
-                elif blocklist_or_passlist == "passlist":
-                    # if the value for that key in the element is in the
-                    # passlist -> Polygon
-                    if key_value in polygon_features_values:
-                        is_polygon = True
+                # become Polygons, and if the value for that key in the
+                # element is in the passlist -> Polygon
+                elif (blocklist_or_passlist == "passlist") and (
+                    key_value in polygon_features_values
+                ):
+                    is_polygon = True
 
     return is_polygon
 
@@ -883,24 +899,25 @@ def _subtract_inner_polygons_from_outer_polygons(element, outer_polygons, inner_
     # loop through the outer polygons subtracting the inner polygons and
     # appending to the list
     for outer_polygon in outer_polygons:
+        outer_polygon_diff = outer_polygon
         for inner_polygon in inner_polygons:
             if inner_polygon.within(outer_polygon):
                 try:
-                    outer_polygon = outer_polygon.difference(inner_polygon)
+                    outer_polygon_diff = outer_polygon_diff.difference(inner_polygon)
                 except TopologicalError:  # pragma: no cover
                     utils.log(
-                        f"relation https://www.openstreetmap.org/relation/{element['id']} caused"
-                        " a TopologicalError, trying with zero buffer."
+                        f"relation https://www.openstreetmap.org/relation/{element['id']} "
+                        "caused a TopologicalError, trying with zero buffer."
                     )
-                    outer_polygon = outer_polygon.buffer(0).difference(inner_polygon.buffer(0))
+                    outer_polygon_diff = outer_polygon.buffer(0).difference(inner_polygon.buffer(0))
 
         # note: .buffer(0) can return either a Polygon or MultiPolygon
         # if it returns a MultiPolygon we need to extract the component
         # sub Polygons to add to outer_polygons_with_holes
-        if outer_polygon.geom_type == "Polygon":
-            outer_polygons_with_holes.append(outer_polygon)
-        elif outer_polygon.geom_type == "MultiPolygon":
-            outer_polygons_with_holes.extend(list(outer_polygon.geoms))
+        if outer_polygon_diff.geom_type == "Polygon":
+            outer_polygons_with_holes.append(outer_polygon_diff)
+        elif outer_polygon_diff.geom_type == "MultiPolygon":
+            outer_polygons_with_holes.extend(list(outer_polygon_diff.geoms))
 
     # if only one polygon with holes was created, return that single polygon
     if len(outer_polygons_with_holes) == 1:
@@ -1025,7 +1042,7 @@ def _filter_gdf_by_polygon_and_tags(gdf, polygon, tags):
         gdf = gdf[polygon_filter & combined_tag_filter].copy()
 
         # remove columns of all nulls (created by discarded component features)
-        gdf.dropna(axis="columns", how="all", inplace=True)
+        gdf = gdf.dropna(axis="columns", how="all")
 
     # multi-index gdf by element_type and osmid then return
     idx_cols = ["element_type", "osmid"]
