@@ -8,19 +8,21 @@ https://nominatim.org/.
 
 import logging as lg
 from collections import OrderedDict
+from warnings import warn
 
 import geopandas as gpd
 import pandas as pd
 
-from . import _downloader
+from . import _nominatim
 from . import projection
 from . import settings
 from . import utils
+from ._errors import InsufficientResponseError
 
 
 def geocode(query):
     """
-    Geocode place names or addresses to (lat, lng) with the Nominatim API.
+    Geocode place names or addresses to (lat, lon) with the Nominatim API.
 
     This geocodes the query via the Nominatim "search" endpoint.
 
@@ -32,7 +34,7 @@ def geocode(query):
     Returns
     -------
     point : tuple
-        the (lat, lng) coordinates returned by the geocoder
+        the (lat, lon) coordinates returned by the geocoder
     """
     # define the parameters
     params = OrderedDict()
@@ -40,17 +42,19 @@ def geocode(query):
     params["limit"] = 1
     params["dedupe"] = 0  # prevent deduping to get precise number of results
     params["q"] = query
-    response_json = _downloader._nominatim_request(params=params)
+    response_json = _nominatim._nominatim_request(params=params)
 
-    # if results were returned, parse lat and lng out of the result
+    # if results were returned, parse lat and lon out of the result
     if response_json and "lat" in response_json[0] and "lon" in response_json[0]:
         lat = float(response_json[0]["lat"])
-        lng = float(response_json[0]["lon"])
-        point = (lat, lng)
+        lon = float(response_json[0]["lon"])
+        point = (lat, lon)
         utils.log(f"Geocoded {query!r} to {point}")
         return point
-    else:
-        raise ValueError(f"Nominatim could not geocode query {query!r}")
+
+    # otherwise we got no results back
+    msg = f"Nominatim could not geocode query {query!r}"
+    raise InsufficientResponseError(msg)
 
 
 def geocode_to_gdf(query, which_result=None, by_osmid=False, buffer_dist=None):
@@ -87,15 +91,24 @@ def geocode_to_gdf(query, which_result=None, by_osmid=False, buffer_dist=None):
     by_osmid : bool
         if True, treat query as an OSM ID lookup rather than text search
     buffer_dist : float
-        distance to buffer around the place geometry, in meters
+        deprecated, do not use
 
     Returns
     -------
     gdf : geopandas.GeoDataFrame
         a GeoDataFrame with one row for each query
     """
+    if buffer_dist is not None:
+        warn(
+            "The buffer_dist argument has been deprecated and will be removed "
+            "in the v2.0.0 release. Buffer your results directly, if desired.",
+            FutureWarning,
+            stacklevel=2,
+        )
+
     if not isinstance(query, (str, dict, list)):  # pragma: no cover
-        raise ValueError("query must be a string or dict or list")
+        msg = "query must be a string or dict or list"
+        raise TypeError(msg)
 
     # if caller passed a list of queries but a scalar which_result value, then
     # turn which_result into a list with same length as query list
@@ -110,12 +123,14 @@ def geocode_to_gdf(query, which_result=None, by_osmid=False, buffer_dist=None):
 
     # ensure same length
     if len(query) != len(which_result):  # pragma: no cover
-        raise ValueError("which_result length must equal query length")
+        msg = "which_result length must equal query length"
+        raise ValueError(msg)
 
     # ensure query type of each item
     for q in query:
         if not isinstance(q, (str, dict)):  # pragma: no cover
-            raise ValueError("each query must be a dict or a string")
+            msg = "each query must be a dict or a string"
+            raise TypeError(msg)
 
     # geocode each query and add to GeoDataFrame as a new row
     gdf = gpd.GeoDataFrame()
@@ -127,7 +142,7 @@ def geocode_to_gdf(query, which_result=None, by_osmid=False, buffer_dist=None):
     gdf = gdf.set_crs(settings.default_crs)
 
     # if buffer_dist was passed in, project the geometry to UTM, buffer it in
-    # meters, then project it back to lat-lng
+    # meters, then project it back to lat-lon
     if buffer_dist is not None and len(gdf) > 0:
         gdf_utm = projection.project_gdf(gdf)
         gdf_utm["geometry"] = gdf_utm["geometry"].buffer(buffer_dist)
@@ -159,19 +174,17 @@ def _geocode_query_to_gdf(query, which_result, by_osmid):
     gdf : geopandas.GeoDataFrame
         a GeoDataFrame with one row containing the result of geocoding
     """
-    if which_result is None:
-        limit = 50
-    else:
-        limit = which_result
+    limit = 50 if which_result is None else which_result
 
-    results = _downloader._retrieve_osm_element(query, by_osmid=by_osmid, limit=limit)
+    results = _nominatim._download_nominatim_element(query, by_osmid=by_osmid, limit=limit)
 
     # choose the right result from the JSON response
     if not results:
         # if no results were returned, raise error
-        raise ValueError(f"Nominatim geocoder returned 0 results for query {query!r}")
+        msg = f"Nominatim geocoder returned 0 results for query {query!r}"
+        raise InsufficientResponseError(msg)
 
-    elif by_osmid:
+    if by_osmid:
         # if searching by OSM ID, always take the first (ie, only) result
         result = results[0]
 
@@ -185,8 +198,8 @@ def _geocode_query_to_gdf(query, which_result, by_osmid):
 
     else:  # pragma: no cover
         # else, we got fewer results than which_result, raise error
-        msg = f"Nominatim geocoder only returned {len(results)} result(s) for query {query!r}"
-        raise ValueError(msg)
+        msg = f"Nominatim returned {len(results)} result(s) but which_result={which_result}"
+        raise InsufficientResponseError(msg)
 
     # if we got a non (Multi)Polygon geometry type (like a point), log warning
     geom_type = result["geojson"]["type"]
@@ -241,4 +254,5 @@ def _get_first_polygon(results, query):
             return result
 
     # if we never found a polygon, throw an error
-    raise ValueError(f"Nominatim could not geocode query {query!r} to polygonal boundaries")
+    msg = f"Nominatim could not geocode query {query!r} to a geometry of type (Multi)Polygon"
+    raise TypeError(msg)
