@@ -1,4 +1,11 @@
-"""Download and create graphs from OpenStreetMap data."""
+"""
+Download and create graphs from OpenStreetMap data.
+
+This module uses filters to query the Overpass API: you can either specify a
+built-in network type or provide your own custom filter with Overpass QL.
+
+Refer to the Getting Started guide for usage limitations.
+"""
 
 import itertools
 from warnings import warn
@@ -7,7 +14,7 @@ import networkx as nx
 from shapely.geometry import MultiPolygon
 from shapely.geometry import Polygon
 
-from . import _downloader
+from . import _overpass
 from . import distance
 from . import geocoder
 from . import osm_xml
@@ -19,20 +26,22 @@ from . import truncate
 from . import utils
 from . import utils_geo
 from . import utils_graph
-from ._errors import EmptyOverpassResponse
+from ._errors import CacheOnlyInterruptError
+from ._errors import InsufficientResponseError
 from ._version import __version__
 
 
 def graph_from_bbox(
-    north,
-    south,
-    east,
-    west,
+    north=None,
+    south=None,
+    east=None,
+    west=None,
+    bbox=None,
     network_type="all_private",
     simplify=True,
     retain_all=False,
     truncate_by_edge=False,
-    clean_periphery=True,
+    clean_periphery=None,
     custom_filter=None,
 ):
     """
@@ -45,13 +54,15 @@ def graph_from_bbox(
     Parameters
     ----------
     north : float
-        northern latitude of bounding box
+        deprecated, do not use
     south : float
-        southern latitude of bounding box
+        deprecated, do not use
     east : float
-        eastern longitude of bounding box
+        deprecated, do not use
     west : float
-        western longitude of bounding box
+        deprecated, do not use
+    bbox : tuple of floats
+        bounding box as (north, south, east, west)
     network_type : string {"all_private", "all", "bike", "drive", "drive_service", "walk"}
         what type of street network to get if custom_filter is None
     simplify : bool
@@ -63,8 +74,7 @@ def graph_from_bbox(
         if True, retain nodes outside bounding box if at least one of node's
         neighbors is within the bounding box
     clean_periphery : bool
-        if True, buffer 500m to get a graph larger than requested, then
-        simplify, then truncate it to requested spatial boundaries
+        deprecated, do not use
     custom_filter : string
         a custom ways filter to be used instead of the network_type presets
         e.g., '["power"~"line"]' or '["highway"~"motorway|trunk"]'. Also pass
@@ -81,8 +91,16 @@ def graph_from_bbox(
     function to automatically make multiple requests: see that function's
     documentation for caveats.
     """
+    if not (north is None and south is None and east is None and west is None):
+        msg = (
+            "The `north`, `south`, `east`, and `west` parameters are deprecated and "
+            "will be removed in the v2.0.0 release. Use the `bbox` parameter instead."
+        )
+        warn(msg, FutureWarning, stacklevel=2)
+        bbox = (north, south, east, west)
+
     # convert bounding box to a polygon
-    polygon = utils_geo.bbox_to_poly(north, south, east, west)
+    polygon = utils_geo.bbox_to_poly(bbox=bbox)
 
     # create graph using this polygon geometry
     G = graph_from_polygon(
@@ -95,7 +113,7 @@ def graph_from_bbox(
         custom_filter=custom_filter,
     )
 
-    utils.log(f"graph_from_bbox returned graph with {len(G)} nodes and {len(G.edges)} edges")
+    utils.log(f"graph_from_bbox returned graph with {len(G):,} nodes and {len(G.edges):,} edges")
     return G
 
 
@@ -107,11 +125,11 @@ def graph_from_point(
     simplify=True,
     retain_all=False,
     truncate_by_edge=False,
-    clean_periphery=True,
+    clean_periphery=None,
     custom_filter=None,
 ):
     """
-    Download and create a graph within some distance of a (lat, lng) point.
+    Download and create a graph within some distance of a (lat, lon) point.
 
     You can use the `settings` module to retrieve a snapshot of historical OSM
     data as of a certain date, or to configure the Overpass server timeout,
@@ -120,7 +138,7 @@ def graph_from_point(
     Parameters
     ----------
     center_point : tuple
-        the (lat, lng) center point around which to construct the graph
+        the (lat, lon) center point around which to construct the graph
     dist : int
         retain only those nodes within this many meters of the center of the
         graph, with distance determined according to dist_type argument
@@ -139,8 +157,7 @@ def graph_from_point(
         if True, retain nodes outside bounding box if at least one of node's
         neighbors is within the bounding box
     clean_periphery : bool,
-        if True, buffer 500m to get a graph larger than requested, then
-        simplify, then truncate it to requested spatial boundaries
+        deprecated, do not use
     custom_filter : string
         a custom ways filter to be used instead of the network_type presets
         e.g., '["power"~"line"]' or '["highway"~"motorway|trunk"]'. Also pass
@@ -158,17 +175,15 @@ def graph_from_point(
     documentation for caveats.
     """
     if dist_type not in {"bbox", "network"}:  # pragma: no cover
-        raise ValueError('dist_type must be "bbox" or "network"')
+        msg = 'dist_type must be "bbox" or "network"'
+        raise ValueError(msg)
 
     # create bounding box from center point and distance in each direction
-    north, south, east, west = utils_geo.bbox_from_point(center_point, dist)
+    bbox = utils_geo.bbox_from_point(center_point, dist)
 
     # create a graph from the bounding box
     G = graph_from_bbox(
-        north,
-        south,
-        east,
-        west,
+        bbox=bbox,
         network_type=network_type,
         simplify=simplify,
         retain_all=retain_all,
@@ -183,7 +198,7 @@ def graph_from_point(
         node = distance.nearest_nodes(G, X=[center_point[1]], Y=[center_point[0]])[0]
         G = truncate.truncate_graph_dist(G, node, max_dist=dist)
 
-    utils.log(f"graph_from_point returned graph with {len(G)} nodes and {len(G.edges)} edges")
+    utils.log(f"graph_from_point returned graph with {len(G):,} nodes and {len(G.edges):,} edges")
     return G
 
 
@@ -195,8 +210,8 @@ def graph_from_address(
     simplify=True,
     retain_all=False,
     truncate_by_edge=False,
-    return_coords=False,
-    clean_periphery=True,
+    return_coords=None,
+    clean_periphery=None,
     custom_filter=None,
 ):
     """
@@ -229,10 +244,9 @@ def graph_from_address(
         if True, retain nodes outside bounding box if at least one of node's
         neighbors is within the bounding box
     return_coords : bool
-        optionally also return the geocoded coordinates of the address
-    clean_periphery : bool,
-        if True, buffer 500m to get a graph larger than requested, then
-        simplify, then truncate it to requested spatial boundaries
+        deprecated, do not use
+    clean_periphery : bool
+        deprecated, do not use
     custom_filter : string
         a custom ways filter to be used instead of the network_type presets
         e.g., '["power"~"line"]' or '["highway"~"motorway|trunk"]'. Also pass
@@ -241,7 +255,7 @@ def graph_from_address(
 
     Returns
     -------
-    networkx.MultiDiGraph or optionally (networkx.MultiDiGraph, (lat, lng))
+    networkx.MultiDiGraph or optionally (networkx.MultiDiGraph, (lat, lon))
 
     Notes
     -----
@@ -249,7 +263,17 @@ def graph_from_address(
     function to automatically make multiple requests: see that function's
     documentation for caveats.
     """
-    # geocode the address string to a (lat, lng) point
+    if return_coords is None:
+        return_coords = False
+    else:
+        warn(
+            "The `return_coords` argument has been deprecated and will be removed in "
+            "the v2.0.0 release. Future behavior will be as though `return_coords=False`. "
+            "If you want the address's geocoded coordinates, use the `geocode` function.",
+            FutureWarning,
+            stacklevel=2,
+        )
+    # geocode the address string to a (lat, lon) point
     point = geocoder.geocode(query=address)
 
     # then create a graph from this point
@@ -264,12 +288,13 @@ def graph_from_address(
         clean_periphery=clean_periphery,
         custom_filter=custom_filter,
     )
-    utils.log(f"graph_from_address returned graph with {len(G)} nodes and {len(G.edges)} edges")
+    utils.log(f"graph_from_address returned graph with {len(G):,} nodes and {len(G.edges):,} edges")
 
     if return_coords:
         return G, point
-    else:
-        return G
+
+    # otherwise
+    return G
 
 
 def graph_from_place(
@@ -280,7 +305,7 @@ def graph_from_place(
     truncate_by_edge=False,
     which_result=None,
     buffer_dist=None,
-    clean_periphery=True,
+    clean_periphery=None,
     custom_filter=None,
 ):
     """
@@ -320,10 +345,9 @@ def graph_from_place(
         which geocoding result to use. if None, auto-select the first
         (Multi)Polygon or raise an error if OSM doesn't return one.
     buffer_dist : float
-        distance to buffer around the place geometry, in meters
+        deprecated, do not use
     clean_periphery : bool
-        if True, buffer 500m to get a graph larger than requested, then
-        simplify, then truncate it to requested spatial boundaries
+        deprecated, do not use
     custom_filter : string
         a custom ways filter to be used instead of the network_type presets
         e.g., '["power"~"line"]' or '["highway"~"motorway|trunk"]'. Also pass
@@ -340,6 +364,14 @@ def graph_from_place(
     function to automatically make multiple requests: see that function's
     documentation for caveats.
     """
+    if buffer_dist is not None:
+        warn(
+            "The buffer_dist argument has been deprecated and will be removed "
+            "in the v2.0.0 release. Buffer your query area directly, if desired.",
+            FutureWarning,
+            stacklevel=2,
+        )
+
     # create a GeoDataFrame with the spatial boundaries of the place(s)
     if isinstance(query, (str, dict)):
         # if it is a string (place name) or dict (structured place query),
@@ -351,7 +383,8 @@ def graph_from_place(
         # if it is a list, it contains multiple places to get
         gdf_place = geocoder.geocode_to_gdf(query, buffer_dist=buffer_dist)
     else:  # pragma: no cover
-        raise TypeError("query must be dict, string, or list of strings")
+        msg = "query must be dict, string, or list of strings"
+        raise TypeError(msg)
 
     # extract the geometry from the GeoDataFrame to use in API query
     polygon = gdf_place["geometry"].unary_union
@@ -368,7 +401,7 @@ def graph_from_place(
         custom_filter=custom_filter,
     )
 
-    utils.log(f"graph_from_place returned graph with {len(G)} nodes and {len(G.edges)} edges")
+    utils.log(f"graph_from_place returned graph with {len(G):,} nodes and {len(G.edges):,} edges")
     return G
 
 
@@ -378,7 +411,7 @@ def graph_from_polygon(
     simplify=True,
     retain_all=False,
     truncate_by_edge=False,
-    clean_periphery=True,
+    clean_periphery=None,
     custom_filter=None,
 ):
     """
@@ -404,8 +437,7 @@ def graph_from_polygon(
         if True, retain nodes outside boundary polygon if at least one of
         node's neighbors is within the polygon
     clean_periphery : bool
-        if True, buffer 500m to get a graph larger than requested, then
-        simplify, then truncate it to requested spatial boundaries
+        deprecated, do not use
     custom_filter : string
         a custom ways filter to be used instead of the network_type presets
         e.g., '["power"~"line"]' or '["highway"~"motorway|trunk"]'. Also pass
@@ -422,17 +454,29 @@ def graph_from_polygon(
     function to automatically make multiple requests: see that function's
     documentation for caveats.
     """
+    if clean_periphery is None:
+        clean_periphery = True
+    else:
+        warn(
+            "The clean_periphery argument has been deprecated and will be removed in "
+            "the v2.0.0 release. Future behavior will be as though clean_periphery=True.",
+            FutureWarning,
+            stacklevel=2,
+        )
+
     # verify that the geometry is valid and is a shapely Polygon/MultiPolygon
     # before proceeding
     if not polygon.is_valid:  # pragma: no cover
-        raise ValueError("The geometry to query within is invalid")
+        msg = "The geometry to query within is invalid"
+        raise ValueError(msg)
     if not isinstance(polygon, (Polygon, MultiPolygon)):  # pragma: no cover
-        raise TypeError(
-            "Geometry must be a shapely Polygon or MultiPolygon. If you requested "
-            "graph from place name, make sure your query resolves to a Polygon or "
-            "MultiPolygon, and not some other geometry, like a Point. See OSMnx "
-            "documentation for details."
+        msg = (
+            "Geometry must be a shapely Polygon or MultiPolygon. If you "
+            "requested graph from place name, make sure your query resolves "
+            "to a Polygon or MultiPolygon, and not some other geometry, like "
+            "a Point. See OSMnx documentation for details."
         )
+        raise TypeError(msg)
 
     if clean_periphery:
         # create a new buffered polygon 0.5km around the desired one
@@ -442,7 +486,9 @@ def graph_from_polygon(
         poly_buff, _ = projection.project_geometry(poly_proj_buff, crs=crs_utm, to_latlong=True)
 
         # download the network data from OSM within buffered polygon
-        response_jsons = _downloader._osm_network_download(poly_buff, network_type, custom_filter)
+        response_jsons = _overpass._download_overpass_network(
+            poly_buff, network_type, custom_filter
+        )
 
         # create buffered graph from the downloaded data
         bidirectional = network_type in settings.bidirectional_network_types
@@ -474,7 +520,7 @@ def graph_from_polygon(
     # if clean_periphery=False, just use the polygon as provided
     else:
         # download the network data from OSM
-        response_jsons = _downloader._osm_network_download(polygon, network_type, custom_filter)
+        response_jsons = _overpass._download_overpass_network(polygon, network_type, custom_filter)
 
         # create graph from the downloaded data
         bidirectional = network_type in settings.bidirectional_network_types
@@ -500,14 +546,21 @@ def graph_from_polygon(
             stacklevel=2,
         )
 
-    utils.log(f"graph_from_polygon returned graph with {len(G)} nodes and {len(G.edges)} edges")
+    utils.log(f"graph_from_polygon returned graph with {len(G):,} nodes and {len(G.edges):,} edges")
     return G
 
 
-def graph_from_xml(filepath, bidirectional=False, simplify=True, retain_all=False):
+def graph_from_xml(
+    filepath, bidirectional=False, simplify=True, retain_all=False, encoding="utf-8"
+):
     """
     Create a graph from data in a .osm formatted XML file.
 
+    Do not load an XML file generated by OSMnx: this use case is not supported
+    and may not behave as expected. To save/load graphs to/from disk for later
+    use in OSMnx, use the `io.save_graphml` and `io.load_graphml` functions
+    instead.
+
     Parameters
     ----------
     filepath : string or pathlib.Path
@@ -519,13 +572,15 @@ def graph_from_xml(filepath, bidirectional=False, simplify=True, retain_all=Fals
     retain_all : bool
         if True, return the entire graph even if it is not connected.
         otherwise, retain only the largest weakly connected component.
+    encoding : string
+        the XML file's character encoding
 
     Returns
     -------
     G : networkx.MultiDiGraph
     """
     # transmogrify file of OSM XML data into JSON
-    response_jsons = [osm_xml._overpass_json_from_file(filepath)]
+    response_jsons = [osm_xml._overpass_json_from_file(filepath, encoding)]
 
     # create graph using this response JSON
     G = _create_graph(response_jsons, bidirectional=bidirectional, retain_all=retain_all)
@@ -534,7 +589,7 @@ def graph_from_xml(filepath, bidirectional=False, simplify=True, retain_all=Fals
     if simplify:
         G = simplification.simplify_graph(G)
 
-    utils.log(f"graph_from_xml returned graph with {len(G)} nodes and {len(G.edges)} edges")
+    utils.log(f"graph_from_xml returned graph with {len(G):,} nodes and {len(G.edges):,} edges")
     return G
 
 
@@ -548,8 +603,8 @@ def _create_graph(response_jsons, retain_all=False, bidirectional=False):
 
     Parameters
     ----------
-    response_jsons : list
-        list of dicts of JSON responses from from the Overpass API
+    response_jsons : iterable
+        iterable of dicts of JSON responses from from the Overpass API
     retain_all : bool
         if True, return the entire graph even if it is not connected.
         otherwise, retain only the largest weakly connected component.
@@ -560,14 +615,35 @@ def _create_graph(response_jsons, retain_all=False, bidirectional=False):
     -------
     G : networkx.MultiDiGraph
     """
-    utils.log("Creating graph from downloaded OSM data...")
+    response_count = 0
+    nodes = {}
+    paths = {}
 
-    # make sure we got data back from the server request(s)
-    if not any(rj["elements"] for rj in response_jsons):  # pragma: no cover
-        msg = "There are no data elements in the server response. Check log and query location/filters."
-        raise EmptyOverpassResponse(msg)
+    # consume response_jsons generator to download data from server
+    for response_json in response_jsons:
+        response_count += 1
 
-    # create the graph as a MultiDiGraph and set its meta-attributes
+        # if cache_only_mode, consume response_jsons then continue next loop
+        if settings.cache_only_mode:  # pragma: no cover
+            continue
+
+        # otherwise, extract nodes and paths from the downloaded OSM data
+        nodes_temp, paths_temp = _parse_nodes_paths(response_json)
+        nodes.update(nodes_temp)
+        paths.update(paths_temp)
+
+    utils.log(f"Retrieved all data from API in {response_count} request(s)")
+    if settings.cache_only_mode:  # pragma: no cover
+        # after consuming all response_jsons in loop, raise exception to catch
+        msg = "Interrupted because `settings.cache_only_mode=True`"
+        raise CacheOnlyInterruptError(msg)
+
+    # ensure we got some node/way data back from the server request(s)
+    if (len(nodes) == 0) and (len(paths) == 0):  # pragma: no cover
+        msg = "No data elements in server response. Check query location/filters and log."
+        raise InsufficientResponseError(msg)
+
+    # create the MultiDiGraph and set its graph-level attributes
     metadata = {
         "created_date": utils.ts(),
         "created_with": f"OSMnx {__version__}",
@@ -575,26 +651,16 @@ def _create_graph(response_jsons, retain_all=False, bidirectional=False):
     }
     G = nx.MultiDiGraph(**metadata)
 
-    # extract nodes and paths from the downloaded osm data
-    nodes = {}
-    paths = {}
-    for response_json in response_jsons:
-        nodes_temp, paths_temp = _parse_nodes_paths(response_json)
-        nodes.update(nodes_temp)
-        paths.update(paths_temp)
-
-    # add each osm node to the graph
-    for node, data in nodes.items():
-        G.add_node(node, **data)
-
-    # add each osm way (ie, a path of edges) to the graph
+    # add each OSM node and way (a path of edges) to the graph
+    utils.log(f"Creating graph from {len(nodes):,} OSM nodes and {len(paths):,} OSM ways...")
+    G.add_nodes_from(nodes.items())
     _add_paths(G, paths.values(), bidirectional)
 
-    # retain only the largest connected component if retain_all is False
+    # retain only the largest connected component if retain_all=False
     if not retain_all:
         G = utils_graph.get_largest_component(G)
 
-    utils.log(f"Created graph with {len(G)} nodes and {len(G.edges)} edges")
+    utils.log(f"Created graph with {len(G):,} nodes and {len(G.edges):,} edges")
 
     # add length (great-circle distance between nodes) attribute to each edge
     if len(G.edges) > 0:
@@ -697,7 +763,7 @@ def _is_path_one_way(path, bidirectional, oneway_values):
         return True
 
     # rule 2
-    elif bidirectional:
+    if bidirectional:
         # if this is a bi-directional network type, then nothing in it is
         # considered one-way. eg, if this is a walking network, this may very
         # well be a one-way street (as cars/bikes go), but in a walking-only
@@ -707,19 +773,18 @@ def _is_path_one_way(path, bidirectional, oneway_values):
         return False
 
     # rule 3
-    elif "oneway" in path and path["oneway"] in oneway_values:
+    if "oneway" in path and path["oneway"] in oneway_values:
         # if this path is tagged as one-way and if it is not a bi-directional
         # network type then we'll add the path in one direction only
         return True
 
     # rule 4
-    elif "junction" in path and path["junction"] == "roundabout":
+    if "junction" in path and path["junction"] == "roundabout":
         # roundabouts are also one-way but are not explicitly tagged as such
         return True
 
-    else:
-        # otherwise this path is not tagged as a one-way
-        return False
+    # otherwise, if no rule passed then this path is not tagged as a one-way
+    return False
 
 
 def _is_path_reversed(path, reversed_values):
@@ -738,10 +803,7 @@ def _is_path_reversed(path, reversed_values):
     -------
     bool
     """
-    if "oneway" in path and path["oneway"] in reversed_values:
-        return True
-    else:
-        return False
+    return "oneway" in path and path["oneway"] in reversed_values
 
 
 def _add_paths(G, paths, bidirectional=False):
