@@ -1,6 +1,7 @@
 """Serialize graphs to/from files on disk."""
 
 import ast
+import contextlib
 from pathlib import Path
 from warnings import warn
 
@@ -37,10 +38,7 @@ def save_graph_geopackage(G, filepath=None, encoding="utf-8", directed=False):
     None
     """
     # default filepath if none was provided
-    if filepath is None:
-        filepath = Path(settings.data_folder) / "graph.gpkg"
-    else:
-        filepath = Path(filepath)
+    filepath = Path(settings.data_folder) / "graph.gpkg" if filepath is None else Path(filepath)
 
     # if save folder does not already exist, create it
     filepath.parent.mkdir(parents=True, exist_ok=True)
@@ -87,16 +85,16 @@ def save_graph_shapefile(G, filepath=None, encoding="utf-8", directed=False):
     """
     warn(
         "The `save_graph_shapefile` function is deprecated and will be removed "
-        "in a future release. Instead, use the `save_graph_geopackage` function "
+        "in the v2.0.0 release. Instead, use the `save_graph_geopackage` function "
         "to save graphs as GeoPackage files for subsequent GIS analysis.",
+        FutureWarning,
         stacklevel=2,
     )
 
     # default filepath if none was provided
-    if filepath is None:
-        filepath = Path(settings.data_folder) / "graph_shapefile"
-    else:
-        filepath = Path(filepath)
+    filepath = (
+        Path(settings.data_folder) / "graph_shapefile" if filepath is None else Path(filepath)
+    )
 
     # if save folder does not already exist, create it (shapefiles
     # get saved as set of files)
@@ -140,10 +138,7 @@ def save_graphml(G, filepath=None, gephi=False, encoding="utf-8"):
     None
     """
     # default filepath if none was provided
-    if filepath is None:
-        filepath = Path(settings.data_folder) / "graph.graphml"
-    else:
-        filepath = Path(filepath)
+    filepath = Path(settings.data_folder) / "graph.graphml" if filepath is None else Path(filepath)
 
     # if save folder does not already exist, create it
     filepath.parent.mkdir(parents=True, exist_ok=True)
@@ -220,7 +215,8 @@ def load_graphml(
     if (filepath is None and graphml_str is None) or (
         filepath is not None and graphml_str is not None
     ):  # pragma: no cover
-        raise ValueError("You must pass one and only one of `filepath` or `graphml_str`.")
+        msg = "You must pass one and only one of `filepath` or `graphml_str`."
+        raise ValueError(msg)
 
     # specify default graph/node/edge attribute values' data types
     default_graph_dtypes = {"simplified": _convert_bool_string}
@@ -419,6 +415,15 @@ def _convert_node_attr_types(G, dtypes=None):
     G : networkx.MultiDiGraph
     """
     for _, data in G.nodes(data=True):
+        # first, eval stringified lists, dicts, or sets to convert them to objects
+        # lists, dicts, or sets would be custom attribute types added by a user
+        for attr, value in data.items():
+            if (value.startswith("[") and value.endswith("]")) or (
+                value.startswith("{") and value.endswith("}")
+            ):
+                with contextlib.suppress(SyntaxError, ValueError):
+                    data[attr] = ast.literal_eval(value)
+
         for attr in data.keys() & dtypes.keys():
             data[attr] = dtypes[attr](data[attr])
     return G
@@ -444,14 +449,15 @@ def _convert_edge_attr_types(G, dtypes=None):
         # remove extraneous "id" attribute added by graphml saving
         data.pop("id", None)
 
-        # first, eval stringified lists to convert them to list objects
+        # first, eval stringified lists, dicts, or sets to convert them to objects
         # edge attributes might have a single value, or a list if simplified
+        # dicts or sets would be custom attribute types added by a user
         for attr, value in data.items():
-            if value.startswith("[") and value.endswith("]"):
-                try:
+            if (value.startswith("[") and value.endswith("]")) or (
+                value.startswith("{") and value.endswith("}")
+            ):
+                with contextlib.suppress(SyntaxError, ValueError):
                     data[attr] = ast.literal_eval(value)
-                except (SyntaxError, ValueError):
-                    pass
 
         # next, convert attribute value types if attribute appears in dtypes
         for attr in data.keys() & dtypes.keys():
@@ -491,10 +497,13 @@ def _convert_bool_string(value):
     """
     if value in {"True", "False"}:
         return value == "True"
-    elif isinstance(value, bool):
+
+    if isinstance(value, bool):
         return value
-    else:  # pragma: no cover
-        raise ValueError(f"invalid literal for boolean: {value!r}")
+
+    # otherwise the value is not a valid boolean
+    msg = f"invalid literal for boolean: {value!r}"
+    raise ValueError(msg)
 
 
 def _stringify_nonnumeric_cols(gdf):
@@ -515,7 +524,7 @@ def _stringify_nonnumeric_cols(gdf):
         gdf with non-numeric columns stringified
     """
     # stringify every non-numeric column other than geometry column
-    for col in (c for c in gdf.columns if not c == "geometry"):
+    for col in (c for c in gdf.columns if c != "geometry"):
         if not pd.api.types.is_numeric_dtype(gdf[col]):
             gdf[col] = gdf[col].fillna("").astype(str)
 
