@@ -1,6 +1,7 @@
 """Visualize street networks, routes, orientations, and geospatial features."""
 
 from pathlib import Path
+from warnings import warn
 
 import networkx as nx
 import numpy as np
@@ -17,12 +18,12 @@ from . import utils_graph
 
 # matplotlib is an optional dependency needed for visualization
 try:
-    import matplotlib.cm as cm
-    import matplotlib.colors as colors
     import matplotlib.pyplot as plt
+    from matplotlib import cm
     from matplotlib import colormaps
+    from matplotlib import colors
 except ImportError:  # pragma: no cover
-    cm = colors = plt = colormaps = None
+    cm = colors = plt = colormaps = None  # type: ignore[assignment]
 
 
 def get_colors(n, cmap="viridis", start=0.0, stop=1.0, alpha=1.0, return_hex=False):
@@ -40,21 +41,37 @@ def get_colors(n, cmap="viridis", start=0.0, stop=1.0, alpha=1.0, return_hex=Fal
     stop : float
         where to end in the colorspace
     alpha : float
-        opacity, the alpha channel for the RGBa colors
+        If `None`, return colors as HTML-like hex triplet "#rrggbb" RGB
+        strings. If `float`, return as "#rrggbbaa" RGBa strings.
     return_hex : bool
-        if True, convert RGBa colors to HTML-like hexadecimal RGB strings. if
-        False, return colors as (R, G, B, alpha) tuples.
+        deprecated, do not use
 
     Returns
     -------
     color_list : list
     """
-    _verify_mpl()
-    color_list = [colormaps[cmap](x) for x in np.linspace(start, stop, n)]
-    if return_hex:
-        color_list = [colors.to_hex(c) for c in color_list]
+    if return_hex is None:
+        return_hex = False
     else:
+        warn(
+            "The `return_hex` parameter has been deprecated and will be removed "
+            "in the v2.0.0 release.",
+            FutureWarning,
+            stacklevel=2,
+        )
+
+    _verify_mpl()
+
+    color_list = [colormaps[cmap](x) for x in np.linspace(start, stop, n)]
+    keep_alpha = alpha is not None
+    if keep_alpha:
         color_list = [(r, g, b, alpha) for r, g, b, _ in color_list]
+    else:
+        color_list = [(r, g, b) for r, g, b, _ in color_list]
+
+    if return_hex:
+        return [colors.to_hex(c, keep_alpha=keep_alpha) for c in color_list]
+
     return color_list
 
 
@@ -208,7 +225,8 @@ def plot_graph(
     max_node_size = max(node_size) if hasattr(node_size, "__iter__") else node_size
     max_edge_lw = max(edge_linewidth) if hasattr(edge_linewidth, "__iter__") else edge_linewidth
     if max_node_size <= 0 and max_edge_lw <= 0:  # pragma: no cover
-        raise ValueError("Either node_size or edge_linewidth must be > 0 to plot something.")
+        msg = "Either node_size or edge_linewidth must be > 0 to plot something."
+        raise ValueError(msg)
 
     # create fig, ax as needed
     utils.log("Begin plotting the graph...")
@@ -296,8 +314,8 @@ def plot_graph_route(
     if ax is None:
         # plot the graph but not the route, and override any user show/close
         # args for now: we'll do that later
-        override = {"show", "save", "close"}
-        kwargs = {k: v for k, v in pg_kwargs.items() if k not in override}
+        overrides = {"show", "save", "close"}
+        kwargs = {k: v for k, v in pg_kwargs.items() if k not in overrides}
         fig, ax = plot_graph(G, show=False, save=False, close=False, **kwargs)
     else:
         fig = ax.figure
@@ -357,21 +375,25 @@ def plot_graph_routes(G, routes, route_colors="r", route_linewidths=4, **pgr_kwa
 
     # check for valid arguments
     if not all(isinstance(r, list) for r in routes):  # pragma: no cover
-        raise ValueError("routes must be a list of route lists")
-    if len(routes) < 2:  # pragma: no cover
-        raise ValueError("You must pass more than 1 route")
+        msg = "routes must be a list of route lists"
+        raise ValueError(msg)
+    if len(routes) <= 1:  # pragma: no cover
+        msg = "You must pass more than 1 route"
+        raise ValueError(msg)
     if isinstance(route_colors, str):
         route_colors = [route_colors] * len(routes)
     if len(routes) != len(route_colors):  # pragma: no cover
-        raise ValueError("route_colors list must have same length as routes")
+        msg = "route_colors list must have same length as routes"
+        raise ValueError(msg)
     if isinstance(route_linewidths, int):
         route_linewidths = [route_linewidths] * len(routes)
     if len(routes) != len(route_linewidths):  # pragma: no cover
-        raise ValueError("route_linewidths list must have same length as routes")
+        msg = "route_linewidths list must have same length as routes"
+        raise ValueError(msg)
 
     # plot the graph and the first route
-    override = {"route", "route_color", "route_linewidth", "show", "save", "close"}
-    kwargs = {k: v for k, v in pgr_kwargs.items() if k not in override}
+    overrides = {"route", "route_color", "route_linewidth", "show", "save", "close"}
+    kwargs = {k: v for k, v in pgr_kwargs.items() if k not in overrides}
     fig, ax = plot_graph_route(
         G,
         route=routes[0],
@@ -384,8 +406,8 @@ def plot_graph_routes(G, routes, route_colors="r", route_linewidths=4, **pgr_kwa
     )
 
     # plot the subsequent routes on top of existing ax
-    override.update({"ax"})
-    kwargs = {k: v for k, v in pgr_kwargs.items() if k not in override}
+    overrides.update({"ax"})
+    kwargs = {k: v for k, v in pgr_kwargs.items() if k not in overrides}
     r_rc_rlw = zip(routes[1:], route_colors[1:], route_linewidths[1:])
     for route, route_color, route_linewidth in r_rc_rlw:
         fig, ax = plot_graph_route(
@@ -415,9 +437,9 @@ def plot_figure_ground(
     network_type="drive_service",
     street_widths=None,
     default_width=4,
-    figsize=(8, 8),
-    edge_color="w",
-    smooth_joints=True,
+    color="w",
+    edge_color=None,
+    smooth_joints=None,
     **pg_kwargs,
 ):
     """
@@ -428,24 +450,23 @@ def plot_figure_ground(
     G : networkx.MultiDiGraph
         input graph, must be unprojected
     address : string
-        address to geocode as the center point if G is not passed in
+        deprecated, do not use
     point : tuple
-        center point if address and G are not passed in
+        deprecated, do not use
     dist : numeric
         how many meters to extend north, south, east, west from center point
     network_type : string
-        what type of street network to get
+        deprecated, do not use
     street_widths : dict
         dict keys are street types and values are widths to plot in pixels
     default_width : numeric
         fallback width in pixels for any street type not in street_widths
-    figsize : numeric
-        (width, height) of figure, should be equal
+    color : string
+        color of the streets
     edge_color : string
-        color of the edges' lines
+        deprecated, do not use
     smooth_joints : bool
-        if True, plot nodes same width as streets to smooth line joints and
-        prevent cracks between them from showing
+        deprecated, do not use
     pg_kwargs
         keyword arguments to pass to plot_graph
 
@@ -455,7 +476,24 @@ def plot_figure_ground(
         matplotlib figure, axis
     """
     _verify_mpl()
-    multiplier = 1.2
+
+    if edge_color is None:
+        edge_color = color
+    else:
+        msg = (
+            "The `edge_color` parameter is deprecated and will be removed in the "
+            "v2.0.0 release. Use `color` instead."
+        )
+        warn(msg, FutureWarning, stacklevel=2)
+
+    if smooth_joints is None:
+        smooth_joints = True
+    else:
+        msg = (
+            "The `smooth_joints` parameter is deprecated and will be removed in the "
+            "v2.0.0 release. In the future this function will behave as though True."
+        )
+        warn(msg, FutureWarning, stacklevel=2)
 
     # if user did not pass in custom street widths, create a dict of defaults
     if street_widths is None:
@@ -469,16 +507,23 @@ def plot_figure_ground(
             "motorway": 6,
         }
 
+    multiplier = 1.2
+    dep_msg = (
+        "The `address`, `point`, and `network_type` parameters are deprecated "
+        "and will be removed in the v2.0.0 release. Pass `G` instead."
+    )
+
     # if G was passed in, plot it centered on its node centroid
     if G is not None:
         gdf_nodes = utils_graph.graph_to_gdfs(G, edges=False, node_geometry=True)
-        lnglat_point = gdf_nodes.unary_union.centroid.coords[0]
-        point = tuple(reversed(lnglat_point))
+        lonlat_point = gdf_nodes.unary_union.centroid.coords[0]
+        point = tuple(reversed(lonlat_point))
 
     # otherwise get network by address or point (whichever was passed) using a
     # dist multiplier to ensure we get more than enough network. simplify in
     # non-strict mode to not combine multiple street types into single edge
     elif address is not None:
+        warn(dep_msg, FutureWarning, stacklevel=2)
         G, point = graph.graph_from_address(
             address,
             dist=dist * multiplier,
@@ -490,6 +535,7 @@ def plot_figure_ground(
         )
         G = simplification.simplify_graph(G, strict=False)
     elif point is not None:
+        warn(dep_msg, FutureWarning, stacklevel=2)
         G = graph.graph_from_point(
             point,
             dist=dist * multiplier,
@@ -500,7 +546,8 @@ def plot_figure_ground(
         )
         G = simplification.simplify_graph(G, strict=False)
     else:  # pragma: no cover
-        raise ValueError("You must pass an address or lat-lng point or graph.")
+        msg = "You must pass an address or lat-lon point or graph."
+        raise ValueError(msg)
 
     # we need an undirected graph to find every edge incident on a node
     Gu = utils_graph.get_undirected(G)
@@ -534,9 +581,7 @@ def plot_figure_ground(
                         et_flat.append(et)
 
                 # lookup corresponding width for each edge type in flat list
-                edge_widths = [
-                    street_widths[et] if et in street_widths else default_width for et in et_flat
-                ]
+                edge_widths = [street_widths.get(et, default_width) for et in et_flat]
 
                 # node diameter should equal largest edge width to make joints
                 # perfectly smooth. alternatively use min(?) to prevent
@@ -555,12 +600,11 @@ def plot_figure_ground(
     bbox = utils_geo.bbox_from_point(point, dist, project_utm=False)
 
     # plot the figure
-    override = {"bbox", "node_size", "node_color", "edge_linewidth"}
-    kwargs = {k: v for k, v in pg_kwargs.items() if k not in override}
+    overrides = {"bbox", "node_size", "node_color", "edge_linewidth"}
+    kwargs = {k: v for k, v in pg_kwargs.items() if k not in overrides}
     fig, ax = plot_graph(
         G=Gu,
         bbox=bbox,
-        figsize=figsize,
         node_size=node_sizes,
         node_color=edge_color,
         edge_color=edge_color,
@@ -685,7 +729,7 @@ def plot_orientation(
         undirected, unprojected graph with `bearing` attributes on each edge
     num_bins : int
         number of bins; for example, if `num_bins=36` is provided, then each
-        bin will represent 10° around the compass
+        bin will represent 10 degrees around the compass
     min_length : float
         ignore edges with `length` attributes less than `min_length`
     weight : string
@@ -736,20 +780,16 @@ def plot_orientation(
     bin_counts, bin_edges = bearing._bearings_distribution(Gu, num_bins, min_length, weight)
 
     # positions: where to center each bar. ignore the last bin edge, because
-    # it's the same as the first (i.e., 0° = 360°)
+    # it's the same as the first (i.e., 0 degrees = 360 degrees)
     positions = np.radians(bin_edges[:-1])
 
     # width: make bars fill the circumference without gaps or overlaps
     width = 2 * np.pi / num_bins
 
-    # radius: how long to make each bar
+    # radius: how long to make each bar. set bar length so either the bar area
+    # (ie, via sqrt) or the bar height is proportional to the bin's frequency
     bin_frequency = bin_counts / bin_counts.sum()
-    if area:
-        # set bar length so area is proportional to frequency
-        radius = np.sqrt(bin_frequency)
-    else:
-        # set bar length so height is proportional to frequency
-        radius = bin_frequency
+    radius = np.sqrt(bin_frequency) if area else bin_frequency
 
     # create ax (if necessary) then set N at top and go clockwise
     if ax is None:
@@ -819,7 +859,8 @@ def _get_colors_by_value(vals, num_bins, cmap, start, stop, na_color, equal_size
         series labels are node/edge IDs and values are colors
     """
     if len(vals) == 0:
-        raise ValueError("There are no attribute values.")
+        msg = "There are no attribute values."
+        raise ValueError(msg)
 
     if num_bins is None:
         # calculate min/max values based on start/stop and data range
@@ -833,14 +874,14 @@ def _get_colors_by_value(vals, num_bins, cmap, start, stop, na_color, equal_size
         normalizer = colors.Normalize(full_min, full_max)
         scalar_mapper = cm.ScalarMappable(normalizer, colormaps[cmap])
         color_series = vals.map(scalar_mapper.to_rgba)
-        color_series.loc[pd.isnull(vals)] = na_color
+        color_series.loc[pd.isna(vals)] = na_color
 
     else:
         # otherwise, bin values then assign colors to bins
         cut_func = pd.qcut if equal_size else pd.cut
         bins = cut_func(vals, num_bins, labels=range(num_bins))
         bin_colors = get_colors(num_bins, cmap, start, stop)
-        color_list = [bin_colors[b] if pd.notnull(b) else na_color for b in bins]
+        color_list = [bin_colors[b] if pd.notna(b) else na_color for b in bins]
         color_series = pd.Series(color_list, index=bins.index)
 
     return color_series
@@ -878,10 +919,7 @@ def _save_and_show(fig, ax, save=False, show=True, close=True, filepath=None, dp
 
     if save:
         # default filepath, if none provided
-        if filepath is None:
-            filepath = Path(settings.imgs_folder) / "image.png"
-        else:
-            filepath = Path(filepath)
+        filepath = Path(settings.imgs_folder) / "image.png" if filepath is None else Path(filepath)
 
         # if save folder does not already exist, create it
         filepath.parent.mkdir(parents=True, exist_ok=True)
@@ -974,6 +1012,5 @@ def _verify_mpl():
     None
     """
     if cm is None or colors is None or plt is None or colormaps is None:  # pragma: no cover
-        raise ImportError(
-            "matplotlib must be installed as an optional dependency for visualization"
-        )
+        msg = "matplotlib must be installed as an optional dependency for visualization"
+        raise ImportError(msg)
