@@ -1,7 +1,6 @@
-"""Reproject a graph, GeoDataFrame, or geometry to a different CRS."""
+"""Project a graph, GeoDataFrame, or geometry to a different CRS."""
 
 import geopandas as gpd
-import numpy as np
 
 from . import settings
 from . import utils
@@ -16,8 +15,8 @@ def is_projected(crs):
     ----------
     crs : string or pyproj.CRS
         the identifier of the coordinate reference system, which can be
-        anything accepted by pyproj.CRS.from_user_input() such as an authority
-        string or a WKT string
+        anything accepted by `pyproj.CRS.from_user_input()` such as an
+        authority string or a WKT string
 
     Returns
     -------
@@ -29,23 +28,25 @@ def is_projected(crs):
 
 def project_geometry(geometry, crs=None, to_crs=None, to_latlong=False):
     """
-    Project a shapely geometry from its current CRS to another.
+    Project a Shapely geometry from its current CRS to another.
 
-    If to_crs is None, project to the UTM CRS for the UTM zone in which the
-    geometry's centroid lies. Otherwise project to the CRS defined by to_crs.
+    If `to_latlong` is `True`, this projects the GeoDataFrame to the CRS
+    defined by `settings.default_crs`, otherwise it projects it to the CRS
+    defined by `to_crs`. If `to_crs` is `None`, it projects it to the CRS of
+    an appropriate UTM zone given `geometry`'s bounds.
 
     Parameters
     ----------
-    geometry : shapely.geometry.Polygon or shapely.geometry.MultiPolygon
-        the geometry to project
+    geometry : shapely geometry
+        the geometry to be projected
     crs : string or pyproj.CRS
-        the starting CRS of the passed-in geometry. if None, it will be set to
-        settings.default_crs
+        the initial CRS of `geometry`. if None, it will be set to
+        `settings.default_crs`
     to_crs : string or pyproj.CRS
-        if None, project to UTM zone in which geometry's centroid lies,
-        otherwise project to this CRS
+        if None, project to an appropriate UTM zone, otherwise project to
+        this CRS
     to_latlong : bool
-        if True, project to settings.default_crs and ignore to_crs
+        if True, project to `settings.default_crs` and ignore `to_crs`
 
     Returns
     -------
@@ -65,21 +66,20 @@ def project_gdf(gdf, to_crs=None, to_latlong=False):
     """
     Project a GeoDataFrame from its current CRS to another.
 
-    If to_crs is None, project to the UTM CRS for the UTM zone in which the
-    GeoDataFrame's centroid lies. Otherwise project to the CRS defined by
-    to_crs. The simple UTM zone calculation in this function works well for
-    most latitudes, but may not work for some extreme northern locations like
-    Svalbard or far northern Norway.
+    If `to_latlong` is `True`, this projects the GeoDataFrame to the CRS
+    defined by `settings.default_crs`, otherwise it projects it to the CRS
+    defined by `to_crs`. If `to_crs` is `None`, it projects it to the CRS of
+    an appropriate UTM zone given `gdf`'s bounds.
 
     Parameters
     ----------
     gdf : geopandas.GeoDataFrame
         the GeoDataFrame to be projected
     to_crs : string or pyproj.CRS
-        if None, project to UTM zone in which gdf's centroid lies, otherwise
-        project to this CRS
+        if None, project to an appropriate UTM zone, otherwise project to
+        this CRS
     to_latlong : bool
-        if True, project to settings.default_crs and ignore to_crs
+        if True, project to `settings.default_crs` and ignore `to_crs`
 
     Returns
     -------
@@ -87,62 +87,55 @@ def project_gdf(gdf, to_crs=None, to_latlong=False):
         the projected GeoDataFrame
     """
     if gdf.crs is None or len(gdf) < 1:  # pragma: no cover
-        raise ValueError("GeoDataFrame must have a valid CRS and cannot be empty")
+        msg = "GeoDataFrame must have a valid CRS and cannot be empty"
+        raise ValueError(msg)
 
-    # if to_latlong is True, project the gdf to latlong
+    # if to_latlong is True, project the gdf to the default_crs
     if to_latlong:
-        gdf_proj = gdf.to_crs(settings.default_crs)
-        utils.log(f"Projected GeoDataFrame to {settings.default_crs}")
+        to_crs = settings.default_crs
 
-    # else if to_crs was passed-in, project gdf to this CRS
-    elif to_crs is not None:
-        gdf_proj = gdf.to_crs(to_crs)
-        utils.log(f"Projected GeoDataFrame to {to_crs}")
-
-    # otherwise, automatically project the gdf to UTM
-    else:
-        if is_projected(gdf.crs):  # pragma: no cover
-            raise ValueError("Geometry must be unprojected to calculate UTM zone")
-
-        # calculate approximate longitude of centroid of union of all geometries in gdf
-        avg_lng = gdf["geometry"].representative_point().x.mean()
-
-        # calculate UTM zone from avg longitude to define CRS to project to
-        utm_zone = int(np.floor((avg_lng + 180) / 6) + 1)
-        utm_crs = f"+proj=utm +zone={utm_zone} +ellps=WGS84 +datum=WGS84 +units=m +no_defs"
-
-        # project the GeoDataFrame to the UTM CRS
-        gdf_proj = gdf.to_crs(utm_crs)
-        utils.log(f"Projected GeoDataFrame to {gdf_proj.crs}")
+    # else if to_crs is None, project gdf to an appropriate UTM zone
+    elif to_crs is None:
+        to_crs = gdf.estimate_utm_crs()
 
+    # project the gdf
+    gdf_proj = gdf.to_crs(to_crs)
+    crs_desc = f"{gdf_proj.crs.to_string()} / {gdf_proj.crs.name}"
+    utils.log(f"Projected GeoDataFrame to {crs_desc!r}")
     return gdf_proj
 
 
-def project_graph(G, to_crs=None):
+def project_graph(G, to_crs=None, to_latlong=False):
     """
-    Project graph from its current CRS to another.
+    Project a graph from its current CRS to another.
 
-    If to_crs is None, project the graph to the UTM CRS for the UTM zone in
-    which the graph's centroid lies. Otherwise, project the graph to the CRS
-    defined by to_crs.
+    If `to_latlong` is `True`, this projects the GeoDataFrame to the CRS
+    defined by `settings.default_crs`, otherwise it projects it to the CRS
+    defined by `to_crs`. If `to_crs` is `None`, it projects it to the CRS of
+    an appropriate UTM zone given `G`'s bounds.
 
     Parameters
     ----------
     G : networkx.MultiDiGraph
         the graph to be projected
     to_crs : string or pyproj.CRS
-        if None, project graph to UTM zone in which graph centroid lies,
-        otherwise project graph to this CRS
+        if None, project to an appropriate UTM zone, otherwise project to
+        this CRS
+    to_latlong : bool
+        if True, project to `settings.default_crs` and ignore `to_crs`
 
     Returns
     -------
     G_proj : networkx.MultiDiGraph
         the projected graph
     """
+    if to_latlong:
+        to_crs = settings.default_crs
+
     # STEP 1: PROJECT THE NODES
     gdf_nodes = utils_graph.graph_to_gdfs(G, edges=False)
 
-    # create new lat/lng columns to preserve lat/lng for later reference if
+    # create new lat/lon columns to preserve lat/lon for later reference if
     # cols do not already exist (ie, don't overwrite in later re-projections)
     if "lon" not in gdf_nodes.columns or "lat" not in gdf_nodes.columns:
         gdf_nodes["lon"] = gdf_nodes["x"]
