@@ -1,6 +1,7 @@
 """Calculate graph edge speeds and travel times."""
 
 import re
+from warnings import warn
 
 import networkx as nx
 import numpy as np
@@ -9,7 +10,7 @@ import pandas as pd
 from . import utils_graph
 
 
-def add_edge_speeds(G, hwy_speeds=None, fallback=None, precision=1, agg=np.mean):
+def add_edge_speeds(G, hwy_speeds=None, fallback=None, precision=None, agg=np.mean):
     """
     Add edge speeds (km per hour) to graph as new `speed_kph` edge attributes.
 
@@ -46,7 +47,7 @@ def add_edge_speeds(G, hwy_speeds=None, fallback=None, precision=1, agg=np.mean)
         type did not appear in `hwy_speeds` and had no preexisting speed
         values on any edge
     precision : int
-        decimal precision to round speed_kph
+        deprecated, do not use
     agg : function
         aggregation function to impute missing values from observed values.
         the default is numpy.mean, but you might also consider for example
@@ -57,6 +58,15 @@ def add_edge_speeds(G, hwy_speeds=None, fallback=None, precision=1, agg=np.mean)
     G : networkx.MultiDiGraph
         graph with speed_kph attributes on all edges
     """
+    if precision is None:
+        precision = 1
+    else:
+        warn(
+            "The `precision` parameter is deprecated and will be removed in the v2.0.0 release.",
+            FutureWarning,
+            stacklevel=2,
+        )
+
     if fallback is None:
         fallback = np.nan
 
@@ -80,10 +90,7 @@ def add_edge_speeds(G, hwy_speeds=None, fallback=None, precision=1, agg=np.mean)
 
     # if user provided hwy_speeds, use them as default values, otherwise
     # initialize an empty series to populate with values
-    if hwy_speeds is None:
-        hwy_speed_avg = pd.Series(dtype=float)
-    else:
-        hwy_speed_avg = pd.Series(hwy_speeds).dropna()
+    hwy_speed_avg = pd.Series(dtype=float) if hwy_speeds is None else pd.Series(hwy_speeds).dropna()
 
     # for each highway type that caller did not provide in hwy_speeds, impute
     # speed of type by taking the mean of the preexisting speed values of that
@@ -105,23 +112,21 @@ def add_edge_speeds(G, hwy_speeds=None, fallback=None, precision=1, agg=np.mean)
 
     # all speeds will be null if edges had no preexisting maxspeed data and
     # caller did not pass in hwy_speeds or fallback arguments
-    if pd.isnull(speed_kph).all():
-        raise ValueError(
-            (
-                "this graph's edges have no preexisting `maxspeed` "
-                "attribute values so you must pass `hwy_speeds` or "
-                "`fallback` arguments."
-            )
+    if pd.isna(speed_kph).all():
+        msg = (
+            "this graph's edges have no preexisting `maxspeed` attribute "
+            "values so you must pass `hwy_speeds` or `fallback` arguments."
         )
+        raise ValueError(msg)
 
     # add speed kph attribute to graph edges
-    edges["speed_kph"] = speed_kph.round(precision).values
+    edges["speed_kph"] = speed_kph.round(precision).to_numpy()
     nx.set_edge_attributes(G, values=edges["speed_kph"], name="speed_kph")
 
     return G
 
 
-def add_edge_travel_times(G, precision=1):
+def add_edge_travel_times(G, precision=None):
     """
     Add edge travel time (seconds) to graph as new `travel_time` edge attributes.
 
@@ -135,23 +140,33 @@ def add_edge_travel_times(G, precision=1):
     G : networkx.MultiDiGraph
         input graph
     precision : int
-        decimal precision to round travel_time
+        deprecated, do not use
 
     Returns
     -------
     G : networkx.MultiDiGraph
         graph with travel_time attributes on all edges
     """
+    if precision is None:
+        precision = 1
+    else:
+        warn(
+            "The `precision` parameter is deprecated and will be removed in the v2.0.0 release.",
+            FutureWarning,
+            stacklevel=2,
+        )
+
     edges = utils_graph.graph_to_gdfs(G, nodes=False)
 
-    # verify edge length and speed_kph attributes exist and contain no nulls
+    # verify edge length and speed_kph attributes exist
     if not ("length" in edges.columns and "speed_kph" in edges.columns):  # pragma: no cover
-        raise KeyError("all edges must have `length` and `speed_kph` attributes.")
-    else:
-        if (
-            pd.isnull(edges["length"]).any() or pd.isnull(edges["speed_kph"]).any()
-        ):  # pragma: no cover
-            raise ValueError("edge `length` and `speed_kph` values must be non-null.")
+        msg = "all edges must have `length` and `speed_kph` attributes."
+        raise KeyError(msg)
+
+    # verify edge length and speed_kph attributes contain no nulls
+    if pd.isna(edges["length"]).any() or pd.isna(edges["speed_kph"]).any():  # pragma: no cover
+        msg = "edge `length` and `speed_kph` values must be non-null."
+        raise ValueError(msg)
 
     # convert distance meters to km, and speed km per hour to km per second
     distance_km = edges["length"] / 1000
@@ -161,38 +176,51 @@ def add_edge_travel_times(G, precision=1):
     travel_time = distance_km / speed_km_sec
 
     # add travel time attribute to graph edges
-    edges["travel_time"] = travel_time.round(precision).values
+    edges["travel_time"] = travel_time.round(precision).to_numpy()
     nx.set_edge_attributes(G, values=edges["travel_time"], name="travel_time")
 
     return G
 
 
-def _clean_maxspeed(value, convert_mph=True):
+def _clean_maxspeed(maxspeed, agg=np.mean, convert_mph=True):
     """
     Clean a maxspeed string and convert mph to kph if necessary.
 
+    If present, splits maxspeed on "|" (which denotes that the value contains
+    different speeds per lane) then aggregates the resulting values. Invalid
+    inputs return None. See https://wiki.openstreetmap.org/wiki/Key:maxspeed
+    for details on values and formats.
+
     Parameters
     ----------
-    value : string
-        an OSM way maxspeed value
+    maxspeed : string
+        a valid OpenStreetMap way maxspeed value
+    agg : function
+        aggregation function if maxspeed contains multiple values (default
+        is numpy.mean)
     convert_mph : bool
-        if True, convert mph to kph
+        if True, convert miles per hour to km per hour
 
     Returns
     -------
-    value_clean : string
+    clean_value : string
     """
-    MPH_TO_KPH = 1.60934
-    pattern = re.compile(r"[^\d\.,;]")
-
+    MILES_TO_KM = 1.60934
+    # regex adapted from OSM wiki
+    pattern = "^([0-9][\\.,0-9]+?)(?:[ ]?(?:km/h|kmh|kph|mph|knots))?$"
+    values = re.split(r"\|", maxspeed)  # creates a list even if it's a single value
     try:
-        # strip out everything but numbers, periods, commas, semicolons
-        value_clean = float(re.sub(pattern, "", value).replace(",", "."))
-        if convert_mph and "mph" in value.lower():
-            value_clean = value_clean * MPH_TO_KPH
-        return value_clean
+        clean_values = []
+        for value in values:
+            match = re.match(pattern, value)
+            clean_value = float(match.group(1).replace(",", "."))
+            if convert_mph and "mph" in maxspeed.lower():
+                clean_value = clean_value * MILES_TO_KM
+            clean_values.append(clean_value)
+        return agg(clean_values)
 
-    except ValueError:
+    except (ValueError, AttributeError):
+        # if invalid input, return None
         return None
 
 
@@ -217,11 +245,11 @@ def _collapse_multiple_maxspeed_values(value, agg):
     if not isinstance(value, list):
         return value
 
-    else:
-        try:
-            # clean each value in list and convert to kph if it is mph then
-            # return a single aggregated value
-            values = [_clean_maxspeed(x) for x in value]
-            return int(agg(pd.Series(values).dropna()))
-        except ValueError:
-            return None
+    # otherwise, if it is a list, process it
+    try:
+        # clean each value in list and convert to kph if it is mph then
+        # return a single aggregated value
+        values = [_clean_maxspeed(x) for x in value]
+        return int(agg(pd.Series(values).dropna()))
+    except ValueError:
+        return None
