@@ -42,7 +42,7 @@ def streets_per_node(G):
     """
     spn = dict(nx.get_node_attributes(G, "street_count"))
     if set(spn) != set(G.nodes):
-        utils.log("Graph nodes changed since `street_count`s were calculated", level=lg.WARN)
+        utils.log("Graph nodes changed since `street_count`s were calculated", level=lg.WARNING)
     return spn
 
 
@@ -143,7 +143,8 @@ def street_segment_count(Gu):
         count of street segments in graph
     """
     if nx.is_directed(Gu):  # pragma: no cover
-        raise ValueError("`Gu` must be undirected")
+        msg = "`Gu` must be undirected"
+        raise ValueError(msg)
     return len(Gu.edges)
 
 
@@ -162,7 +163,8 @@ def street_length_total(Gu):
         total length (meters) of streets in graph
     """
     if nx.is_directed(Gu):  # pragma: no cover
-        raise ValueError("`Gu` must be undirected")
+        msg = "`Gu` must be undirected"
+        raise ValueError(msg)
     return sum(d["length"] for u, v, d in Gu.edges(data=True))
 
 
@@ -200,7 +202,8 @@ def self_loop_proportion(Gu):
         proportion of graph edges that are self-loops
     """
     if nx.is_directed(Gu):  # pragma: no cover
-        raise ValueError("`Gu` must be undirected")
+        msg = "`Gu` must be undirected"
+        raise ValueError(msg)
     return sum(u == v for u, v, k in Gu.edges) / len(Gu.edges)
 
 
@@ -223,7 +226,8 @@ def circuity_avg(Gu):
         the graph's average undirected edge circuity
     """
     if nx.is_directed(Gu):  # pragma: no cover
-        raise ValueError("`Gu` must be undirected")
+        msg = "`Gu` must be undirected"
+        raise ValueError(msg)
 
     # extract the edges' endpoint nodes' coordinates
     coords = np.array(
@@ -240,9 +244,9 @@ def circuity_avg(Gu):
     # calculate straight-line distances as euclidean distances if projected or
     # great-circle distances if unprojected
     if projection.is_projected(Gu.graph["crs"]):
-        sl_dists = distance.euclidean_dist_vec(y1=y1, x1=x1, y2=y2, x2=x2)
+        sl_dists = distance.euclidean(y1=y1, x1=x1, y2=y2, x2=x2)
     else:
-        sl_dists = distance.great_circle_vec(lat1=y1, lng1=x1, lat2=y2, lng2=x2)
+        sl_dists = distance.great_circle(lat1=y1, lon1=x1, lat2=y2, lon2=x2)
 
     # return the ratio, handling possible division by zero
     sl_dists_total = sl_dists[~np.isnan(sl_dists)].sum()
@@ -315,8 +319,8 @@ def basic_stats(G, area=None, clean_int_tol=None):
     G : networkx.MultiDiGraph
         input graph
     area : float
-        if not None, calculate density measures and use this area value (in
-        square meters) as the denominator
+        if not None, calculate density measures and use this value (in square
+        meters) as the denominator
     clean_int_tol : float
         if not None, calculate consolidated intersections count (and density,
         if `area` is also provided) and use this tolerance value; refer to the
@@ -326,7 +330,7 @@ def basic_stats(G, area=None, clean_int_tol=None):
     Returns
     -------
     stats : dict
-        dictionary containing the following attributes
+        dictionary containing the following keys
           - `circuity_avg` - see `circuity_avg` function documentation
           - `clean_intersection_count` - see `clean_intersection_count` function documentation
           - `clean_intersection_density_km` - `clean_intersection_count` per sq km
@@ -349,7 +353,7 @@ def basic_stats(G, area=None, clean_int_tol=None):
           - `streets_per_node_proportions` - see `streets_per_node_proportions` function documentation
     """
     Gu = utils_graph.get_undirected(G)
-    stats = dict()
+    stats = {}
 
     stats["n"] = len(G.nodes)
     stats["m"] = len(G.edges)
