@@ -1,5 +1,7 @@
 """Truncate graph by distance, bounding box, or polygon."""
 
+from warnings import warn
+
 import networkx as nx
 
 from . import utils
@@ -19,14 +21,12 @@ def truncate_graph_dist(G, source_node, max_dist=1000, weight="length", retain_a
     G : networkx.MultiDiGraph
         input graph
     source_node : int
-        the node in the graph from which to measure network distances to other
-        nodes
-    max_dist : int
-        remove every node in the graph greater than this distance from the
-        source_node (along the network)
+        node in graph from which to measure network distances to other nodes
+    max_dist : float
+        remove every node in the graph that is greater than this distance (in
+        same units as `weight` attribute) along the network from `source_node`
     weight : string
-        how to weight the graph when measuring distance (default 'length' is
-        how many meters long the edge is)
+        graph edge attribute to use to measure distance
     retain_all : bool
         if True, return the entire graph even if it is not connected.
         otherwise, retain only the largest weakly connected component.
@@ -59,14 +59,15 @@ def truncate_graph_dist(G, source_node, max_dist=1000, weight="length", retain_a
 
 def truncate_graph_bbox(
     G,
-    north,
-    south,
-    east,
-    west,
+    north=None,
+    south=None,
+    east=None,
+    west=None,
+    bbox=None,
     truncate_by_edge=False,
     retain_all=False,
-    quadrat_width=0.05,
-    min_num=3,
+    quadrat_width=None,
+    min_num=None,
 ):
     """
     Remove every node in graph that falls outside a bounding box.
@@ -76,35 +77,41 @@ def truncate_graph_bbox(
     G : networkx.MultiDiGraph
         input graph
     north : float
-        northern latitude of bounding box
+        deprecated, do not use
     south : float
-        southern latitude of bounding box
+        deprecated, do not use
     east : float
-        eastern longitude of bounding box
+        deprecated, do not use
     west : float
-        western longitude of bounding box
+        deprecated, do not use
+    bbox : tuple of floats
+        bounding box as (north, south, east, west)
     truncate_by_edge : bool
         if True, retain nodes outside bounding box if at least one of node's
         neighbors is within the bounding box
     retain_all : bool
         if True, return the entire graph even if it is not connected.
         otherwise, retain only the largest weakly connected component.
-    quadrat_width : numeric
-        passed on to intersect_index_quadrats: the linear length (in degrees) of
-        the quadrats with which to cut up the geometry (default = 0.05, approx
-        4km at NYC's latitude)
+    quadrat_width : float
+        deprecated, do not use
     min_num : int
-        passed on to intersect_index_quadrats: the minimum number of linear
-        quadrat lines (e.g., min_num=3 would produce a quadrat grid of 4
-        squares)
+        deprecated, do not use
 
     Returns
     -------
     G : networkx.MultiDiGraph
         the truncated graph
     """
+    if not (north is None and south is None and east is None and west is None):
+        msg = (
+            "The `north`, `south`, `east`, and `west` parameters are deprecated and "
+            "will be removed in the v2.0.0 release. Use the `bbox` parameter instead."
+        )
+        warn(msg, FutureWarning, stacklevel=2)
+        bbox = (north, south, east, west)
+
     # convert bounding box to a polygon, then truncate
-    polygon = utils_geo.bbox_to_poly(north, south, east, west)
+    polygon = utils_geo.bbox_to_poly(bbox=bbox)
     G = truncate_graph_polygon(
         G,
         polygon,
@@ -119,7 +126,7 @@ def truncate_graph_bbox(
 
 
 def truncate_graph_polygon(
-    G, polygon, retain_all=False, truncate_by_edge=False, quadrat_width=0.05, min_num=3
+    G, polygon, retain_all=False, truncate_by_edge=False, quadrat_width=None, min_num=None
 ):
     """
     Remove every node in graph that falls outside a (Multi)Polygon.
@@ -136,29 +143,34 @@ def truncate_graph_polygon(
     truncate_by_edge : bool
         if True, retain nodes outside boundary polygon if at least one of
         node's neighbors is within the polygon
-    quadrat_width : numeric
-        passed on to intersect_index_quadrats: the linear length (in degrees)
-        of the quadrats with which to cut up the geometry (default = 0.05,
-        approx 4km at NYC's latitude)
+    quadrat_width : float
+        deprecated, do not use
     min_num : int
-        passed on to intersect_index_quadrats: the minimum number of linear
-        quadrat lines (e.g., min_num=3 would produce a quadrat grid of 4
-        squares)
+        deprecated, do not use
 
     Returns
     -------
     G : networkx.MultiDiGraph
         the truncated graph
     """
+    if quadrat_width is not None or min_num is not None:
+        warn(
+            "The `quadrat_width` and `min_num` parameters are deprecated and "
+            "will be removed in the v2.0.0 release.",
+            FutureWarning,
+            stacklevel=2,
+        )
+
     utils.log("Identifying all nodes that lie outside the polygon...")
 
     # first identify all nodes whose point geometries lie within the polygon
     gs_nodes = utils_graph.graph_to_gdfs(G, edges=False)[["geometry"]]
-    to_keep = utils_geo._intersect_index_quadrats(gs_nodes, polygon, quadrat_width, min_num)
+    to_keep = utils_geo._intersect_index_quadrats(gs_nodes, polygon)
 
     if not to_keep:
         # no graph nodes within the polygon: can't create a graph from that
-        raise ValueError("Found no graph nodes within the requested polygon")
+        msg = "Found no graph nodes within the requested polygon"
+        raise ValueError(msg)
 
     # now identify all nodes whose point geometries lie outside the polygon
     gs_nodes_outside_poly = gs_nodes[~gs_nodes.index.isin(to_keep)]
@@ -181,7 +193,7 @@ def truncate_graph_polygon(
     # make a copy to not mutate original graph object caller passed in
     G = G.copy()
     G.remove_nodes_from(nodes_to_remove)
-    utils.log(f"Removed {len(nodes_to_remove)} nodes outside polygon")
+    utils.log(f"Removed {len(nodes_to_remove):,} nodes outside polygon")
 
     if not retain_all:
         # remove any isolated nodes and retain only the largest component
