@@ -1,6 +1,6 @@
 """Geospatial utility functions."""
 
-import warnings
+from warnings import warn
 
 import networkx as nx
 import numpy as np
@@ -30,8 +30,9 @@ def sample_points(G, n):
     Parameters
     ----------
     G : networkx.MultiGraph
-        graph to sample points from; should be undirected (to not oversample
-        bidirectional edges) and projected (for accurate point interpolation)
+        graph from which to sample points. should be undirected (to avoid
+        oversampling bidirectional edges) and projected (for accurate point
+        interpolation)
     n : int
         how many points to sample
 
@@ -42,12 +43,12 @@ def sample_points(G, n):
         which each point was drawn
     """
     if nx.is_directed(G):  # pragma: no cover
-        warnings.warn("graph should be undirected to not oversample bidirectional edges")
+        warn("graph should be undirected to avoid oversampling bidirectional edges", stacklevel=2)
     gdf_edges = utils_graph.graph_to_gdfs(G, nodes=False)[["geometry", "length"]]
     weights = gdf_edges["length"] / gdf_edges["length"].sum()
-    idx = np.random.choice(gdf_edges.index, size=n, p=weights)
+    idx = np.random.default_rng().choice(gdf_edges.index, size=n, p=weights)
     lines = gdf_edges.loc[idx, "geometry"]
-    return lines.interpolate(np.random.rand(n), normalized=True)
+    return lines.interpolate(np.random.default_rng().random(n), normalized=True)
 
 
 def interpolate_points(geom, dist):
@@ -63,12 +64,12 @@ def interpolate_points(geom, dist):
         a LineString geometry
     dist : float
         spacing distance between interpolated points, in same units as `geom`.
-        smaller values generate more points.
+        smaller values accordingly generate more points.
 
     Yields
     ------
     points : generator
-        a generator of (x, y) tuples of the interpolated points' coordinates
+        tuples of (x, y) floats of the interpolated points' coordinates
     """
     if isinstance(geom, LineString):
         num_vert = max(round(geom.length / dist), 1)
@@ -76,7 +77,8 @@ def interpolate_points(geom, dist):
             point = geom.interpolate(n / num_vert, normalized=True)
             yield point.x, point.y
     else:  # pragma: no cover
-        raise TypeError(f"unhandled geometry type {geom.geom_type}")
+        msg = f"unhandled geometry type {geom.geom_type}"
+        raise TypeError(msg)
 
 
 def _round_polygon_coords(p, precision):
@@ -197,42 +199,50 @@ def _round_multilinestring_coords(mls, precision):
 
 def round_geometry_coords(geom, precision):
     """
-    Round the coordinates of a shapely geometry to some decimal precision.
+    Do not use: deprecated.
 
     Parameters
     ----------
     geom : shapely.geometry.geometry {Point, MultiPoint, LineString, MultiLineString, Polygon, MultiPolygon}
-        the geometry to round the coordinates of
+        deprecated, do not use
     precision : int
-        decimal precision to round coordinates to
+        deprecated, do not use
 
     Returns
     -------
     shapely.geometry.geometry
     """
+    warn(
+        "The `round_geometry_coords` function is deprecated and will be "
+        "removed in the v2.0.0 release.",
+        FutureWarning,
+        stacklevel=2,
+    )
+
     if isinstance(geom, Point):
         return _round_point_coords(geom, precision)
 
-    elif isinstance(geom, MultiPoint):
+    if isinstance(geom, MultiPoint):
         return _round_multipoint_coords(geom, precision)
 
-    elif isinstance(geom, LineString):
+    if isinstance(geom, LineString):
         return _round_linestring_coords(geom, precision)
 
-    elif isinstance(geom, MultiLineString):
+    if isinstance(geom, MultiLineString):
         return _round_multilinestring_coords(geom, precision)
 
-    elif isinstance(geom, Polygon):
+    if isinstance(geom, Polygon):
         return _round_polygon_coords(geom, precision)
 
-    elif isinstance(geom, MultiPolygon):
+    if isinstance(geom, MultiPolygon):
         return _round_multipolygon_coords(geom, precision)
 
-    else:  # pragma: no cover
-        raise TypeError(f"cannot round coordinates of unhandled geometry type: {type(geom)}")
+    # otherwise
+    msg = f"cannot round coordinates of unhandled geometry type: {type(geom)}"
+    raise TypeError(msg)
 
 
-def _consolidate_subdivide_geometry(geometry, max_query_area_size=None):
+def _consolidate_subdivide_geometry(geometry):
     """
     Consolidate and subdivide some geometry.
 
@@ -248,36 +258,39 @@ def _consolidate_subdivide_geometry(geometry, max_query_area_size=None):
     Parameters
     ----------
     geometry : shapely.geometry.Polygon or shapely.geometry.MultiPolygon
-        the geometry to consolidate and subdivide
-    max_query_area_size : int
-        maximum area for any part of the geometry in meters: any polygon
-        bigger than this will get divided up for multiple queries to API
-        (default 50km x 50km). if None, use settings.max_query_area_size
+        the projected (in meter units) geometry to consolidate and subdivide
 
     Returns
     -------
     geometry : shapely.geometry.MultiPolygon
     """
-    if max_query_area_size is None:
-        max_query_area_size = settings.max_query_area_size
-
-    # let the linear length of the quadrats (with which to subdivide the
-    # geometry) be the square root of max area size
-    quadrat_width = np.sqrt(max_query_area_size)
-
     if not isinstance(geometry, (Polygon, MultiPolygon)):  # pragma: no cover
-        raise TypeError("Geometry must be a shapely Polygon or MultiPolygon")
+        msg = "Geometry must be a shapely Polygon or MultiPolygon"
+        raise TypeError(msg)
 
     # if geometry is either 1) a Polygon whose area exceeds the max size, or
     # 2) a MultiPolygon, then get the convex hull around the geometry
+    mqas = settings.max_query_area_size
     if isinstance(geometry, MultiPolygon) or (
-        isinstance(geometry, Polygon) and geometry.area > max_query_area_size
+        isinstance(geometry, Polygon) and geometry.area > mqas
     ):
         geometry = geometry.convex_hull
 
-    # if geometry area exceeds max size, subdivide it into smaller sub-polygons
-    if geometry.area > max_query_area_size:
-        geometry = _quadrat_cut_geometry(geometry, quadrat_width=quadrat_width)
+    # warn user if they passed a geometry with area much larger than max size
+    ratio = int(geometry.area / mqas)
+    warning_threshold = 10
+    if ratio > warning_threshold:
+        msg = (
+            f"This area is {ratio:,} times your configured Overpass max query "
+            "area size. It will automatically be divided up into multiple "
+            "sub-queries accordingly. This may take a long time."
+        )
+        warn(msg, stacklevel=2)
+
+    # if geometry area exceeds max size, subdivide it into smaller subpolygons
+    # that are no greater than settings.max_query_area_size in size
+    if geometry.area > mqas:
+        geometry = _quadrat_cut_geometry(geometry, quadrat_width=np.sqrt(mqas))
 
     if isinstance(geometry, Polygon):
         geometry = MultiPolygon([geometry])
@@ -285,45 +298,7 @@ def _consolidate_subdivide_geometry(geometry, max_query_area_size=None):
     return geometry
 
 
-def _get_polygons_coordinates(geometry):
-    """
-    Extract exterior coordinates from polygon(s) to pass to OSM.
-
-    Ignore the interior ("holes") coordinates.
-
-    Parameters
-    ----------
-    geometry : shapely.geometry.Polygon or shapely.geometry.MultiPolygon
-        the geometry to extract exterior coordinates from
-
-    Returns
-    -------
-    polygon_coord_strs : list
-    """
-    if not isinstance(geometry, MultiPolygon):  # pragma: no cover
-        raise TypeError("Geometry must be a shapely MultiPolygon")
-
-    # extract geometry's exterior coords
-    polygons_coords = []
-    for polygon in geometry.geoms:
-        x, y = polygon.exterior.xy
-        polygons_coords.append(list(zip(x, y)))
-
-    # convert exterior coords to the string format the API expects
-    polygon_coord_strs = []
-    for coords in polygons_coords:
-        s = ""
-        separator = " "
-        for coord in list(coords):
-            # round floating point lats and longs to 6 decimals (ie, ~100 mm)
-            # so we can hash and cache strings consistently
-            s = f"{s}{separator}{coord[1]:.6f}{separator}{coord[0]:.6f}"
-        polygon_coord_strs.append(s.strip(separator))
-
-    return polygon_coord_strs
-
-
-def _quadrat_cut_geometry(geometry, quadrat_width, min_num=3):
+def _quadrat_cut_geometry(geometry, quadrat_width):
     """
     Split a Polygon or MultiPolygon up into sub-polygons of a specified size.
 
@@ -331,17 +306,17 @@ def _quadrat_cut_geometry(geometry, quadrat_width, min_num=3):
     ----------
     geometry : shapely.geometry.Polygon or shapely.geometry.MultiPolygon
         the geometry to split up into smaller sub-polygons
-    quadrat_width : numeric
-        the linear width of the quadrats with which to cut up the geometry (in
-        the units the geometry is in)
-    min_num : int
-        the minimum number of linear quadrat lines (e.g., min_num=3 would
-        produce a quadrat grid of 4 squares)
+    quadrat_width : float
+        width (in geometry's units) of quadrat squares with which to split up
+        the geometry
 
     Returns
     -------
     geometry : shapely.geometry.MultiPolygon
     """
+    # min number of dividing lines (3 produces a grid of 4 quadrat squares)
+    min_num = 3
+
     # create n evenly spaced points between the min and max x and y bounds
     west, south, east, north = geometry.bounds
     x_num = int(np.ceil((east - west) / quadrat_width) + 1)
@@ -356,7 +331,6 @@ def _quadrat_cut_geometry(geometry, quadrat_width, min_num=3):
 
     # recursively split the geometry by each quadrat line
     geometries = [geometry]
-
     for line in lines:
         # split polygon by line if they intersect, otherwise just keep it
         split_geoms = [split(g, line).geoms if g.intersects(line) else [g] for g in geometries]
@@ -366,9 +340,9 @@ def _quadrat_cut_geometry(geometry, quadrat_width, min_num=3):
     return MultiPolygon(geometries)
 
 
-def _intersect_index_quadrats(geometries, polygon, quadrat_width=0.05, min_num=3):
+def _intersect_index_quadrats(geometries, polygon):
     """
-    Identify geometries that intersect a (multi)polygon.
+    Identify geometries that intersect a (Multi)Polygon.
 
     Uses an r-tree spatial index and cuts polygon up into smaller sub-polygons
     for r-tree acceleration. Ensure that geometries and polygon are in the
@@ -380,52 +354,50 @@ def _intersect_index_quadrats(geometries, polygon, quadrat_width=0.05, min_num=3
         the geometries to intersect with the polygon
     polygon : shapely.geometry.Polygon or shapely.geometry.MultiPolygon
         the polygon to intersect with the geometries
-    quadrat_width : numeric
-        linear length (in polygon's units) of quadrat lines with which to cut
-        up the polygon (default = 0.05 degrees, approx 4km at NYC's latitude)
-    min_num : int
-        the minimum number of linear quadrat lines (e.g., min_num=3 would
-        produce a quadrat grid of 4 squares)
 
     Returns
     -------
     geoms_in_poly : set
-        index labels of geometries that intersected polygon
+        set of the index labels of the geometries that intersected the polygon
     """
     # create an r-tree spatial index for the geometries
-    sindex = geometries.sindex
-    utils.log(f"Created r-tree spatial index for {len(geometries)} geometries")
+    rtree = geometries.sindex
+    utils.log(f"Built r-tree spatial index for {len(geometries):,} geometries")
 
-    # cut the polygon into chunks for spatial index intersecting
-    multipoly = _quadrat_cut_geometry(polygon, quadrat_width=quadrat_width, min_num=min_num)
-    geoms_in_poly = set()
+    # cut polygon into chunks for faster spatial index intersecting. specify a
+    # sensible quadrat_width to balance performance (eg, 0.1 degrees is approx
+    # 8 km at NYC's latitude) with either projected or unprojected coordinates
+    quadrat_width = max(0.1, np.sqrt(polygon.area) / 10)
+    multipoly = _quadrat_cut_geometry(polygon, quadrat_width)
+    utils.log(f"Accelerating r-tree with {len(multipoly.geoms)} quadrats")
 
     # loop through each chunk of the polygon to find intersecting geometries
+    # first find approximate matches with spatial index, then precise matches
+    # from those approximate ones
+    geoms_in_poly = set()
     for poly in multipoly.geoms:
-        # first find approximate matches with spatial index, then precise
-        # matches from those approximate ones
-        poly = poly.buffer(0)
-        if poly.is_valid and poly.area > 0:
-            possible_matches_iloc = sindex.intersection(poly.bounds)
+        poly_buff = poly.buffer(0)
+        if poly_buff.is_valid and poly_buff.area > 0:
+            possible_matches_iloc = rtree.intersection(poly_buff.bounds)
             possible_matches = geometries.iloc[list(possible_matches_iloc)]
-            precise_matches = possible_matches[possible_matches.intersects(poly)]
+            precise_matches = possible_matches[possible_matches.intersects(poly_buff)]
             geoms_in_poly.update(precise_matches.index)
 
-    utils.log(f"Identified {len(geoms_in_poly)} geometries inside polygon")
+    utils.log(f"Identified {len(geoms_in_poly):,} geometries inside polygon")
     return geoms_in_poly
 
 
 def bbox_from_point(point, dist=1000, project_utm=False, return_crs=False):
     """
-    Create a bounding box from a (lat, lng) center point.
+    Create a bounding box around a (lat, lon) point.
 
-    Create a bounding box some distance in each direction (north, south, east,
-    and west) from the center point and optionally project it.
+    Create a bounding box some distance (in meters) in each direction (north,
+    south, east, and west) from the center point and optionally project it.
 
     Parameters
     ----------
     point : tuple
-        the (lat, lng) center point to create the bounding box around
+        the (lat, lon) center point to create the bounding box around
     dist : int
         bounding box distance in meters from the center point
     project_utm : bool
@@ -435,49 +407,60 @@ def bbox_from_point(point, dist=1000, project_utm=False, return_crs=False):
 
     Returns
     -------
-    tuple
-        (north, south, east, west) or (north, south, east, west, crs_proj)
+    bbox or bbox, crs: tuple or tuple, crs
+        (north, south, east, west) or ((north, south, east, west), crs)
     """
-    earth_radius = 6_371_009  # meters
-    lat, lng = point
+    EARTH_RADIUS_M = 6_371_009  # meters
+    lat, lon = point
 
-    delta_lat = (dist / earth_radius) * (180 / np.pi)
-    delta_lng = (dist / earth_radius) * (180 / np.pi) / np.cos(lat * np.pi / 180)
+    delta_lat = (dist / EARTH_RADIUS_M) * (180 / np.pi)
+    delta_lon = (dist / EARTH_RADIUS_M) * (180 / np.pi) / np.cos(lat * np.pi / 180)
     north = lat + delta_lat
     south = lat - delta_lat
-    east = lng + delta_lng
-    west = lng - delta_lng
+    east = lon + delta_lon
+    west = lon - delta_lon
 
     if project_utm:
-        bbox_poly = bbox_to_poly(north, south, east, west)
+        bbox_poly = bbox_to_poly(bbox=(north, south, east, west))
         bbox_proj, crs_proj = projection.project_geometry(bbox_poly)
         west, south, east, north = bbox_proj.bounds
 
     utils.log(f"Created bbox {dist} m from {point}: {north},{south},{east},{west}")
 
     if project_utm and return_crs:
-        return north, south, east, west, crs_proj
-    else:
-        return north, south, east, west
+        return (north, south, east, west), crs_proj
+
+    # otherwise
+    return north, south, east, west
 
 
-def bbox_to_poly(north, south, east, west):
+def bbox_to_poly(north=None, south=None, east=None, west=None, bbox=None):
     """
     Convert bounding box coordinates to shapely Polygon.
 
     Parameters
     ----------
     north : float
-        northern coordinate
+        deprecated, do not use
     south : float
-        southern coordinate
+        deprecated, do not use
     east : float
-        eastern coordinate
+        deprecated, do not use
     west : float
-        western coordinate
+        deprecated, do not use
+    bbox : tuple of floats
+        bounding box as (north, south, east, west)
 
     Returns
     -------
     shapely.geometry.Polygon
     """
+    if not (north is None and south is None and east is None and west is None):
+        msg = (
+            "The `north`, `south`, `east`, and `west` parameters are deprecated and "
+            "will be removed in the v2.0.0 release. Use the `bbox` parameter instead."
+        )
+        warn(msg, FutureWarning, stacklevel=2)
+    else:
+        north, south, east, west = bbox
     return Polygon([(west, south), (east, south), (east, north), (west, north)])
