@@ -42,23 +42,25 @@ def graph_to_gdfs(G, nodes=True, edges=True, node_geometry=True, fill_edge_geome
 
     if nodes:
         if not G.nodes:  # pragma: no cover
-            raise ValueError("graph contains no nodes")
+            msg = "graph contains no nodes"
+            raise ValueError(msg)
 
-        nodes, data = zip(*G.nodes(data=True))
+        uvk, data = zip(*G.nodes(data=True))
 
         if node_geometry:
             # convert node x/y attributes to Points for geometry column
-            geom = (Point(d["x"], d["y"]) for d in data)
-            gdf_nodes = gpd.GeoDataFrame(data, index=nodes, crs=crs, geometry=list(geom))
+            node_geoms = (Point(d["x"], d["y"]) for d in data)
+            gdf_nodes = gpd.GeoDataFrame(data, index=uvk, crs=crs, geometry=list(node_geoms))
         else:
-            gdf_nodes = gpd.GeoDataFrame(data, index=nodes)
+            gdf_nodes = gpd.GeoDataFrame(data, index=uvk)
 
-        gdf_nodes.index.rename("osmid", inplace=True)
+        gdf_nodes.index = gdf_nodes.index.rename("osmid")
         utils.log("Created nodes GeoDataFrame from graph")
 
     if edges:
         if not G.edges:  # pragma: no cover
-            raise ValueError("graph contains no edges")
+            msg = "Graph contains no edges"
+            raise ValueError(msg)
 
         u, v, k, data = zip(*G.edges(keys=True, data=True))
 
@@ -68,14 +70,15 @@ def graph_to_gdfs(G, nodes=True, edges=True, node_geometry=True, fill_edge_geome
             x_lookup = nx.get_node_attributes(G, "x")
             y_lookup = nx.get_node_attributes(G, "y")
 
-            def _make_geom(u, v, data, x=x_lookup, y=y_lookup):
+            def _make_edge_geometry(u, v, data, x=x_lookup, y=y_lookup):
                 if "geometry" in data:
                     return data["geometry"]
-                else:
-                    return LineString((Point((x[u], y[u])), Point((x[v], y[v]))))
 
-            geom = map(_make_geom, u, v, data)
-            gdf_edges = gpd.GeoDataFrame(data, crs=crs, geometry=list(geom))
+                # otherwise
+                return LineString((Point((x[u], y[u])), Point((x[v], y[v]))))
+
+            edge_geoms = map(_make_edge_geometry, u, v, data)
+            gdf_edges = gpd.GeoDataFrame(data, crs=crs, geometry=list(edge_geoms))
 
         else:
             gdf_edges = gpd.GeoDataFrame(data)
@@ -88,18 +91,22 @@ def graph_to_gdfs(G, nodes=True, edges=True, node_geometry=True, fill_edge_geome
         gdf_edges["u"] = u
         gdf_edges["v"] = v
         gdf_edges["key"] = k
-        gdf_edges.set_index(["u", "v", "key"], inplace=True)
+        gdf_edges = gdf_edges.set_index(["u", "v", "key"])
 
         utils.log("Created edges GeoDataFrame from graph")
 
     if nodes and edges:
         return gdf_nodes, gdf_edges
-    elif nodes:
+
+    if nodes:
         return gdf_nodes
-    elif edges:
+
+    if edges:
         return gdf_edges
-    else:  # pragma: no cover
-        raise ValueError("you must request nodes or edges or both")
+
+    # otherwise
+    msg = "you must request nodes or edges or both"
+    raise ValueError(msg)
 
 
 def graph_from_gdfs(gdf_nodes, gdf_edges, graph_attrs=None):
@@ -133,7 +140,8 @@ def graph_from_gdfs(gdf_nodes, gdf_edges, graph_attrs=None):
     G : networkx.MultiDiGraph
     """
     if not ("x" in gdf_nodes.columns and "y" in gdf_nodes.columns):  # pragma: no cover
-        raise ValueError("gdf_nodes must contain x and y columns")
+        msg = "gdf_nodes must contain x and y columns"
+        raise ValueError(msg)
 
     # if gdf_nodes has a geometry attribute set, drop that column (as we use x
     # and y for geometry information) and warn the user if the geometry values
@@ -142,7 +150,8 @@ def graph_from_gdfs(gdf_nodes, gdf_edges, graph_attrs=None):
         try:
             all_x_match = (gdf_nodes.geometry.x == gdf_nodes["x"]).all()
             all_y_match = (gdf_nodes.geometry.y == gdf_nodes["y"]).all()
-            assert all_x_match and all_y_match
+            assert all_x_match
+            assert all_y_match
         except (AssertionError, ValueError):  # pragma: no cover
             # AssertionError if x/y coords don't match geometry column
             # ValueError if geometry column contains non-point geometry types
@@ -161,9 +170,9 @@ def graph_from_gdfs(gdf_nodes, gdf_edges, graph_attrs=None):
     # add edges and their attributes to graph, but filter out null attribute
     # values so that edges only get attributes with non-null values
     attr_names = gdf_edges.columns.to_list()
-    for (u, v, k), attr_vals in zip(gdf_edges.index, gdf_edges.values):
+    for (u, v, k), attr_vals in zip(gdf_edges.index, gdf_edges.to_numpy()):
         data_all = zip(attr_names, attr_vals)
-        data = {name: val for name, val in data_all if isinstance(val, list) or pd.notnull(val)}
+        data = {name: val for name, val in data_all if isinstance(val, list) or pd.notna(val)}
         G.add_edge(u, v, key=k, **data)
 
     # add any nodes with no incident edges, since they wouldn't be added above
@@ -228,7 +237,8 @@ def get_route_edge_attributes(
     """
     warn(
         "The `get_route_edge_attributes` function has been deprecated and will "
-        "be removed in a future release. Use the `route_to_gdf` function instead.",
+        "be removed in the v2.0.0 release. Use the `route_to_gdf` function instead.",
+        FutureWarning,
         stacklevel=2,
     )
     attribute_values = []
@@ -266,7 +276,7 @@ def remove_isolated_nodes(G):
     # get the set of all isolated nodes, then remove them
     isolated_nodes = {node for node, degree in G.degree() if degree < 1}
     G.remove_nodes_from(isolated_nodes)
-    utils.log(f"Removed {len(isolated_nodes)} isolated nodes")
+    utils.log(f"Removed {len(isolated_nodes):,} isolated nodes")
     return G
 
 
@@ -303,7 +313,7 @@ def get_largest_component(G, strongly=False):
 
         # induce (frozen) subgraph then unfreeze it by making new MultiDiGraph
         G = nx.MultiDiGraph(G.subgraph(largest_cc))
-        utils.log(f"Got largest {kind} connected component ({len(G)} of {n} total nodes)")
+        utils.log(f"Got largest {kind} connected component ({len(G):,} of {n:,} total nodes)")
 
     return G
 
@@ -472,8 +482,8 @@ def _is_same_geometry(ls1, ls2):
     # reverse the first LineString's coordinates' direction
     geom1_r = [tuple(reversed(coords)) for coords in ls1.xy]
 
-    # if first geometry matches second in either direction, return True
-    return geom1 == geom2 or geom1_r == geom2
+    # if second geometry matches first in either direction, return True
+    return geom2 in (geom1, geom1_r)  # noqa: PLR6201
 
 
 def _update_edge_keys(G):
