@@ -1,6 +1,7 @@
+# ruff: noqa: E402 F841 PLR2004
 """Test suite for the package."""
 
-# use agg backend so you don't need a display on ci
+# use agg backend so you don't need a display on CI
 # do this first before pyplot is imported by anything
 import matplotlib as mpl
 
@@ -20,7 +21,9 @@ import networkx as nx
 import numpy as np
 import pandas as pd
 import pytest
+from requests.exceptions import ConnectionError
 from shapely import wkt
+from shapely.geometry import GeometryCollection
 from shapely.geometry import LineString
 from shapely.geometry import MultiLineString
 from shapely.geometry import MultiPoint
@@ -52,20 +55,39 @@ polygon = wkt.loads(p)
 
 
 def test_logging():
-    # test OSMnx's logger
+    """Test the logger."""
     ox.log("test a fake default message")
     ox.log("test a fake debug", level=lg.DEBUG)
     ox.log("test a fake info", level=lg.INFO)
     ox.log("test a fake warning", level=lg.WARNING)
     ox.log("test a fake error", level=lg.ERROR)
 
-    ox.citation()
+    ox.citation(style="apa")
+    ox.citation(style="bibtex")
+    ox.citation(style="ieee")
     ox.ts(style="date")
     ox.ts(style="time")
 
 
+def test_exceptions():
+    """Test the custom errors."""
+    message = "testing exception"
+
+    with pytest.raises(ox._errors.ResponseStatusCodeError):
+        raise ox._errors.ResponseStatusCodeError(message)
+
+    with pytest.raises(ox._errors.CacheOnlyInterruptError):
+        raise ox._errors.CacheOnlyInterruptError(message)
+
+    with pytest.raises(ox._errors.InsufficientResponseError):
+        raise ox._errors.InsufficientResponseError(message)
+
+    with pytest.raises(ox._errors.GraphSimplificationError):
+        raise ox._errors.GraphSimplificationError(message)
+
+
 def test_coords_rounding():
-    # test the rounding of geometry coordinates
+    """Test the rounding of geometry coordinates."""
     precision = 3
 
     shape1 = Point(1.123456, 2.123456)
@@ -97,16 +119,24 @@ def test_coords_rounding():
     )
     shape2 = ox.utils_geo.round_geometry_coords(shape1, precision)
 
+    with pytest.raises(TypeError, match="cannot round coordinates of unhandled geometry type"):
+        ox.utils_geo.round_geometry_coords(GeometryCollection(), precision)
 
-def test_geocode_to_gdf():
-    # test loading spatial boundaries and plotting
+
+def test_geocoder():
+    """Test retrieving elements by place name and OSM ID."""
     city = ox.geocode_to_gdf("R2999176", by_osmid=True)
     city = ox.geocode_to_gdf(place1, which_result=1, buffer_dist=100)
     city = ox.geocode_to_gdf(place2)
     city_projected = ox.project_gdf(city, to_crs="epsg:3395")
 
+    # test geocoding a bad query: should raise exception
+    with pytest.raises(ox._errors.InsufficientResponseError):
+        _ = ox.geocode("!@#$%^&*")
+
 
 def test_stats():
+    """Test generating graph stats."""
     # create graph, add a new node, add bearings, project it
     G = ox.graph_from_place(place1, network_type="all")
     G.add_node(0, x=location_point[1], y=location_point[0])
@@ -144,6 +174,7 @@ def test_stats():
 
 
 def test_osm_xml():
+    """Test working with .osm XML data."""
     # test loading a graph from a local .osm xml file
     node_id = 53098262
     neighbor_ids = 53092170, 53060438, 53027353, 667744075
@@ -161,9 +192,9 @@ def test_osm_xml():
             edge_key = (node_id, neighbor_id, 0)
             assert neighbor_id in G.nodes
             assert edge_key in G.edges
-            assert G.edges[edge_key]["name"] in ("8th Street", "Willow Street")
+            assert G.edges[edge_key]["name"] in {"8th Street", "Willow Street"}
 
-    os.remove(temp_filename)
+    Path.unlink(Path(temp_filename))
 
     # test .osm xml saving
     default_all_oneway = ox.settings.all_oneway
@@ -172,15 +203,15 @@ def test_osm_xml():
     ox.save_graph_xml(G, merge_edges=False, filepath=Path(ox.settings.data_folder) / "graph.osm")
 
     # test osm xml output merge edges
-    ox.save_graph_xml(G, merge_edges=True, edge_tag_aggs=[("length", "sum")])
+    ox.io.save_graph_xml(G, merge_edges=True, edge_tag_aggs=[("length", "sum")], precision=5)
 
     # test osm xml output from gdfs
     nodes, edges = ox.graph_to_gdfs(G)
     ox.osm_xml.save_graph_xml([nodes, edges])
 
     # test ordered nodes from way
-    df = pd.DataFrame({"u": [54, 2, 5, 3, 10, 19, 20], "v": [76, 3, 8, 10, 5, 20, 15]})
-    ordered_nodes = ox.osm_xml._get_unique_nodes_ordered_from_way(df)
+    df_uv = pd.DataFrame({"u": [54, 2, 5, 3, 10, 19, 20], "v": [76, 3, 8, 10, 5, 20, 15]})
+    ordered_nodes = ox.osm_xml._get_unique_nodes_ordered_from_way(df_uv)
     assert ordered_nodes == [2, 3, 10, 5, 8]
 
     # test roundabout handling
@@ -201,15 +232,33 @@ def test_osm_xml():
 
 
 def test_elevation():
+    """Test working with elevation data."""
     G = ox.graph_from_address(address=address, dist=500, dist_type="bbox", network_type="bike")
-    rasters = list(Path("tests/input_data").glob("elevation*.tif"))
+
+    # add node elevations from Google (fails without API key)
+    with pytest.raises(ox._errors.InsufficientResponseError):
+        _ = ox.elevation.add_node_elevations_google(
+            G,
+            api_key="",
+            max_locations_per_batch=350,
+            precision=2,
+            url_template=ox.settings.elevation_url_template,
+        )
+
+    # add node elevations from Open Topo Data (works without API key)
+    ox.settings.elevation_url_template = (
+        "https://api.opentopodata.org/v1/aster30m?locations={locations}&key={key}"
+    )
+    _ = ox.elevation.add_node_elevations_google(G, batch_size=100, pause=0.01)
 
     # add node elevations from a single raster file (some nodes will be null)
+    rasters = list(Path("tests/input_data").glob("elevation*.tif"))
     G = ox.elevation.add_node_elevations_raster(G, rasters[0], cpus=1)
+    assert pd.notna(pd.Series(dict(G.nodes(data="elevation")))).any()
 
-    # add node elevations from multiple raster files
+    # add node elevations from multiple raster files (no nodes should be null)
     G = ox.elevation.add_node_elevations_raster(G, rasters)
-    assert pd.notnull(pd.Series(dict(G.nodes(data="elevation")))).all()
+    assert pd.notna(pd.Series(dict(G.nodes(data="elevation")))).all()
 
     # add edge grades and their absolute values
     G = ox.add_edge_grades(G, add_absolute=True)
@@ -217,6 +266,7 @@ def test_elevation():
 
 
 def test_routing():
+    """Test working with speed, travel time, and routing."""
     G = ox.graph_from_address(address=address, dist=500, dist_type="bbox", network_type="bike")
 
     # give each edge speed and travel time attributes
@@ -235,6 +285,12 @@ def test_routing():
     assert ox.speed._clean_maxspeed("signal") is None
     assert ox.speed._clean_maxspeed("100;70") is None
 
+    # test collapsing multiple mph values to single kph value
+    assert ox.speed._collapse_multiple_maxspeed_values(["25 mph", "30 mph"], np.mean) == 44
+
+    # test collapsing invalid values: should return None
+    assert ox.speed._collapse_multiple_maxspeed_values(["mph", "kph"], np.mean) is None
+
     orig_x = np.array([-122.404771])
     dest_x = np.array([-122.401429])
     orig_y = np.array([37.794302])
@@ -243,15 +299,19 @@ def test_routing():
     dest_node = ox.distance.nearest_nodes(G, dest_x, dest_y)[0]
 
     # test non-numeric weight (should raise ValueError)
-    try:
+    with pytest.raises(ValueError, match="contains non-numeric values"):
         route = ox.shortest_path(G, orig_node, dest_node, weight="highway")
-        raise AssertionError()
-    except ValueError:
-        pass
+
+    # mismatch iterable and non-iterable orig/dest should raise ValueError
+    msg = "orig and dest must either both be iterable or neither must be iterable"
+    with pytest.raises(ValueError, match=msg):
+        route = ox.shortest_path(G, orig_node, [dest_node])
+
     # test missing weight (should raise warning)
     route = ox.shortest_path(G, orig_node, dest_node, weight="time")
     # test good weight
     route = ox.shortest_path(G, orig_node, dest_node, weight="travel_time")
+    route = ox.distance.shortest_path(G, orig_node, dest_node, weight="travel_time")
 
     route_edges = ox.utils_graph.route_to_gdf(G, route, "travel_time")
     attributes = ox.utils_graph.get_route_edge_attributes(G, route)
@@ -262,8 +322,8 @@ def test_routing():
     # test multiple origins-destinations
     n = 5
     nodes = np.array(G.nodes)
-    origs = np.random.choice(nodes, size=n, replace=True)
-    dests = np.random.choice(nodes, size=n, replace=True)
+    origs = np.random.default_rng().choice(nodes, size=n, replace=True)
+    dests = np.random.default_rng().choice(nodes, size=n, replace=True)
     paths1 = ox.shortest_path(G, origs, dests, weight="length", cpus=1)
     paths2 = ox.shortest_path(G, origs, dests, weight="length", cpus=2)
     paths3 = ox.shortest_path(G, origs, dests, weight="length", cpus=None)
@@ -271,8 +331,13 @@ def test_routing():
 
     # test k shortest paths
     routes = ox.k_shortest_paths(G, orig_node, dest_node, k=2, weight="travel_time")
+    routes = ox.distance.k_shortest_paths(G, orig_node, dest_node, k=2, weight="travel_time")
     fig, ax = ox.plot_graph_routes(G, list(routes))
 
+    # test great circle and euclidean distance calculators
+    assert ox.distance.great_circle_vec(0, 0, 1, 1) == pytest.approx(157249.6034105)
+    assert ox.distance.euclidean_dist_vec(0, 0, 1, 1) == pytest.approx(1.4142135)
+
     # test folium with keyword arguments to pass to folium.PolyLine
     gm = ox.plot_graph_folium(G, popup_attribute="name", color="#333333", weight=5, opacity=0.7)
     rm = ox.plot_route_folium(G, route, color="#cc0000", weight=5, opacity=0.7)
@@ -287,8 +352,10 @@ def test_routing():
 
 
 def test_plots():
+    """Test visualization methods."""
     G = ox.graph_from_point(location_point, dist=500, network_type="drive")
     Gp = ox.project_graph(G)
+    G = ox.project_graph(G, to_latlong=True)
 
     # test getting colors
     co = ox.plot.get_colors(n=5, return_hex=True)
@@ -323,12 +390,13 @@ def test_plots():
 
 
 def test_find_nearest():
+    """Test nearest node/edge searching."""
     # get graph and x/y coords to search
     G = ox.graph_from_point(location_point, dist=500, network_type="drive", simplify=False)
     Gp = ox.project_graph(G)
     points = ox.utils_geo.sample_points(ox.get_undirected(Gp), 5)
-    X = points.x.values
-    Y = points.y.values
+    X = points.x.to_numpy()
+    Y = points.y.to_numpy()
 
     # get nearest nodes
     nn0, dist0 = ox.distance.nearest_nodes(G, X[0], Y[0], return_dist=True)
@@ -340,10 +408,18 @@ def test_find_nearest():
     ne2 = ox.distance.nearest_edges(G, X[0], Y[0], interpolate=50, return_dist=True)
 
 
-def test_endpoints():
+def test_api_endpoints():
+    """Test different API endpoints."""
+    default_timeout = ox.settings.timeout
+    default_key = ox.settings.nominatim_key
+    default_nominatim_endpoint = ox.settings.nominatim_endpoint
+    default_overpass_endpoint = ox.settings.overpass_endpoint
+    default_overpass_rate_limit = ox.settings.overpass_rate_limit
+
+    # test good and bad DNS resolution
+    ox.settings.timeout = 1
     ip = ox._downloader._resolve_host_via_doh("overpass-api.de")
     ip = ox._downloader._resolve_host_via_doh("AAAAAAAAAAA")
-
     _doh_url_template_default = ox.settings.doh_url_template
     ox.settings.doh_url_template = "http://aaaaaa.hostdoesntexist.org/nothinguseful"
     ip = ox._downloader._resolve_host_via_doh("overpass-api.de")
@@ -351,17 +427,27 @@ def test_endpoints():
     ip = ox._downloader._resolve_host_via_doh("overpass-api.de")
     ox.settings.doh_url_template = _doh_url_template_default
 
+    # Test changing the Overpass endpoint.
+    # This should fail because we didn't provide a valid endpoint
+    ox.settings.overpass_rate_limit = False
+    ox.settings.overpass_endpoint = "http://NOT_A_VALID_ENDPOINT/api/"
+    with pytest.raises(ConnectionError, match="Max retries exceeded with url"):
+        G = ox.graph_from_place(place1, network_type="all")
+
+    ox.settings.overpass_rate_limit = default_overpass_rate_limit
+    ox.settings.timeout = default_timeout
+
     params = OrderedDict()
     params["format"] = "json"
     params["address_details"] = 0
 
     # Bad Address - should return an empty response
     params["q"] = "AAAAAAAAAAA"
-    response_json = ox._downloader._nominatim_request(params=params, request_type="search")
+    response_json = ox._nominatim._nominatim_request(params=params, request_type="search")
 
     # Good Address - should return a valid response with a valid osm_id
     params["q"] = "Newcastle A186 Westgate Rd"
-    response_json = ox._downloader._nominatim_request(params=params, request_type="search")
+    response_json = ox._nominatim._nominatim_request(params=params, request_type="search")
 
     # Lookup
     params = OrderedDict()
@@ -369,25 +455,15 @@ def test_endpoints():
     params["address_details"] = 0
     params["osm_ids"] = "W68876073"
 
-    response_json = ox._downloader._nominatim_request(params=params, request_type="lookup")
+    response_json = ox._nominatim._nominatim_request(params=params, request_type="lookup")
 
     # Invalid nominatim query type
-    with pytest.raises(ValueError):
-        response_json = ox._downloader._nominatim_request(params=params, request_type="xyz")
+    with pytest.raises(ValueError, match="Nominatim request_type must be"):
+        response_json = ox._nominatim._nominatim_request(params=params, request_type="xyz")
 
-    default_key = ox.settings.nominatim_key
-    default_nominatim_endpoint = ox.settings.nominatim_endpoint
-    default_overpass_endpoint = ox.settings.overpass_endpoint
-
-    # Searching on public nominatim should work even if a key was provided
+    # Searching on public nominatim should work even if a (bad) key was provided
     ox.settings.nominatim_key = "NOT_A_KEY"
-    response_json = ox._downloader._nominatim_request(params=params, request_type="search")
-
-    # Test changing the endpoint.
-    # This should fail because we didn't provide a valid endpoint
-    ox.settings.overpass_endpoint = "http://NOT_A_VALID_ENDPOINT/api/"
-    with pytest.raises(Exception) as ex:
-        G = ox.graph_from_place(place1, network_type="all")
+    response_json = ox._nominatim._nominatim_request(params=params, request_type="search")
 
     ox.settings.nominatim_key = default_key
     ox.settings.nominatim_endpoint = default_nominatim_endpoint
@@ -395,6 +471,9 @@ def test_endpoints():
 
 
 def test_graph_save_load():
+    """Test saving/loading graphs to/from disk."""
+    fp = Path(ox.settings.data_folder) / "graph.graphml"
+
     # save graph as shapefile and geopackage
     G = ox.graph_from_point(location_point, dist=500, network_type="drive")
     ox.save_graph_shapefile(G, directed=True)
@@ -412,23 +491,44 @@ def test_graph_save_load():
     assert set(gdf_nodes1.index) == set(gdf_nodes2.index) == set(G.nodes) == set(G2.nodes)
     assert set(gdf_edges1.index) == set(gdf_edges2.index) == set(G.edges) == set(G2.edges)
 
+    # test code branches that should raise exceptions
+    with pytest.raises(ValueError, match="you must request nodes or edges or both"):
+        ox.graph_to_gdfs(G2, nodes=False, edges=False)
+    with pytest.raises(ValueError, match="invalid literal for boolean"):
+        ox.io._convert_bool_string("T")
+
     # create random boolean graph/node/edge attributes
     attr_name = "test_bool"
     G.graph[attr_name] = False
-    bools = np.random.randint(0, 2, len(G.nodes))
+    bools = np.random.default_rng().integers(low=0, high=2, size=len(G.nodes))
     node_attrs = {n: bool(b) for n, b in zip(G.nodes, bools)}
     nx.set_node_attributes(G, node_attrs, attr_name)
-    bools = np.random.randint(0, 2, len(G.edges))
+    bools = np.random.default_rng().integers(low=0, high=2, size=len(G.edges))
     edge_attrs = {n: bool(b) for n, b in zip(G.edges, bools)}
     nx.set_edge_attributes(G, edge_attrs, attr_name)
 
+    # create list, set, and dict attributes for nodes and edges
+    rand_ints_nodes = np.random.default_rng().integers(low=0, high=10, size=len(G.nodes))
+    rand_ints_edges = np.random.default_rng().integers(low=0, high=10, size=len(G.edges))
+    list_node_attrs = {n: [n, r] for n, r in zip(G.nodes, rand_ints_nodes)}
+    nx.set_node_attributes(G, list_node_attrs, "test_list")
+    list_edge_attrs = {e: [e, r] for e, r in zip(G.edges, rand_ints_edges)}
+    nx.set_edge_attributes(G, list_edge_attrs, "test_list")
+    set_node_attrs = {n: {n, r} for n, r in zip(G.nodes, rand_ints_nodes)}
+    nx.set_node_attributes(G, set_node_attrs, "test_set")
+    set_edge_attrs = {e: {e, r} for e, r in zip(G.edges, rand_ints_edges)}
+    nx.set_edge_attributes(G, set_edge_attrs, "test_set")
+    dict_node_attrs = {n: {n: r} for n, r in zip(G.nodes, rand_ints_nodes)}
+    nx.set_node_attributes(G, dict_node_attrs, "test_dict")
+    dict_edge_attrs = {e: {e: r} for e, r in zip(G.edges, rand_ints_edges)}
+    nx.set_edge_attributes(G, dict_edge_attrs, "test_dict")
+
     # save/load graph as graphml file
     ox.save_graphml(G, gephi=True)
     ox.save_graphml(G, gephi=False)
-    ox.save_graphml(G, gephi=False, filepath=Path(ox.settings.data_folder) / "graph.graphml")
-    filepath = Path(ox.settings.data_folder) / "graph.graphml"
+    ox.save_graphml(G, gephi=False, filepath=fp)
     G2 = ox.load_graphml(
-        filepath,
+        fp,
         graph_dtypes={attr_name: ox.io._convert_bool_string},
         node_dtypes={attr_name: ox.io._convert_bool_string},
         edge_dtypes={attr_name: ox.io._convert_bool_string},
@@ -453,15 +553,21 @@ def test_graph_save_load():
     # test custom data types
     nd = {"osmid": str}
     ed = {"length": str, "osmid": float}
-    G2 = ox.load_graphml(filepath, node_dtypes=nd, edge_dtypes=ed)
+    G2 = ox.load_graphml(fp, node_dtypes=nd, edge_dtypes=ed)
 
     # test loading graphml from a file stream
-    file_bytes = open("tests/input_data/short.graphml", "rb").read()
+    file_bytes = Path.open(Path("tests/input_data/short.graphml"), "rb").read()
     data = str(file_bytes.decode())
     G = ox.load_graphml(graphml_str=data, node_dtypes=nd, edge_dtypes=ed)
 
 
 def test_graph_from_functions():
+    """Test downloading graphs from Overpass."""
+    # test subdividing a large geometry (raises a UserWarning)
+    bbox = ox.utils_geo.bbox_from_point((0, 0), dist=1e5, project_utm=True)
+    poly = ox.utils_geo.bbox_to_poly(*bbox)
+    _ = ox.utils_geo._consolidate_subdivide_geometry(poly)
+
     # graph from bounding box
     _ = ox.utils_geo.bbox_from_point(location_point, project_utm=True, return_crs=True)
     north, south, east, west = ox.utils_geo.bbox_from_point(location_point, dist=500)
@@ -472,14 +578,14 @@ def test_graph_from_functions():
 
     # truncate graph by bounding box
     north, south, east, west = ox.utils_geo.bbox_from_point(location_point, dist=400)
-    G = ox.truncate.truncate_graph_bbox(G, north, south, east, west)
+    G = ox.truncate.truncate_graph_bbox(G, north, south, east, west, min_num=3)
     G = ox.utils_graph.get_largest_component(G, strongly=True)
 
     # graph from address
     G = ox.graph_from_address(address=address, dist=500, dist_type="bbox", network_type="bike")
 
     # graph from list of places
-    G = ox.graph_from_place([place1], network_type="all", clean_periphery=False)
+    G = ox.graph_from_place([place1], network_type="all", buffer_dist=0, clean_periphery=False)
 
     # graph from polygon
     G = ox.graph_from_polygon(polygon, network_type="walk", truncate_by_edge=True, simplify=False)
@@ -508,11 +614,10 @@ def test_graph_from_functions():
 
 
 def test_features():
+    """Test downloading features from Overpass."""
     # geometries_from_bbox - bounding box query to return no data
-    try:
+    with pytest.raises(ox._errors.InsufficientResponseError):
         gdf = ox.geometries_from_bbox(0.009, -0.009, 0.009, -0.009, tags={"building": True})
-    except ox._errors.EmptyOverpassResponse:
-        pass
 
     # geometries_from_bbox - successful
     north, south, east, west = ox.utils_geo.bbox_from_point(location_point, dist=500)
@@ -526,7 +631,7 @@ def test_features():
 
     # geometries_from_place - includes test of list of places
     tags = {"amenity": True, "landuse": ["retail", "commercial"], "highway": "bus_stop"}
-    gdf = ox.geometries_from_place(place1, tags=tags)
+    gdf = ox.geometries_from_place(place1, tags=tags, buffer_dist=0)
     gdf = ox.geometries_from_place([place1], tags=tags)
 
     # geometries_from_polygon
@@ -547,5 +652,5 @@ def test_features():
         os.close(handle)
     for filename in ("tests/input_data/West-Oakland.osm.bz2", temp_filename):
         gdf = ox.geometries_from_xml(filename)
-        assert "Willow Street" in gdf["name"].values
-    os.remove(temp_filename)
+        assert "Willow Street" in gdf["name"].to_numpy()
+    Path.unlink(Path(temp_filename))
